Struct BoolVector3D
pub struct BoolVector3D {
pub x: bool,
pub y: bool,
pub z: bool,
}geometry only.Expand description
A 3d vector of booleans, useful for component-wise logic operations.
Fields
x: booly: boolz: boolImplementations
impl BoolVector3D
impl BoolVector3D
pub fn all(self) -> bool
Available on crate feature canvas only.
pub fn all(self) -> bool
canvas only.Returns true if all components are true and false otherwise.
pub fn any(self) -> bool
Available on crate feature canvas only.
pub fn any(self) -> bool
canvas only.Returns true if any component are true and false otherwise.
pub fn none(self) -> bool
Available on crate feature canvas only.
pub fn none(self) -> bool
canvas only.Returns true if all components are false and false otherwise. Negation of any().
pub fn and(self, other: BoolVector3D) -> BoolVector3D
Available on crate feature canvas only.
pub fn and(self, other: BoolVector3D) -> BoolVector3D
canvas only.Returns new vector with by-component AND operation applied.
pub fn or(self, other: BoolVector3D) -> BoolVector3D
Available on crate feature canvas only.
pub fn or(self, other: BoolVector3D) -> BoolVector3D
canvas only.Returns new vector with by-component OR operation applied.
pub fn not(self) -> BoolVector3D
Available on crate feature canvas only.
pub fn not(self) -> BoolVector3D
canvas only.Returns new vector with results of negation operation on each component.
pub fn select_point<T, U>(
self,
a: Point3D<T, U>,
b: Point3D<T, U>,
) -> Point3D<T, U>
Available on crate feature canvas only.
pub fn select_point<T, U>( self, a: Point3D<T, U>, b: Point3D<T, U>, ) -> Point3D<T, U>
canvas only.Returns point, each component of which or from a, or from b depending on truly value
of corresponding vector component. true selects value from a and false from b.
pub fn select_vector<T, U>(
self,
a: Vector3D<T, U>,
b: Vector3D<T, U>,
) -> Vector3D<T, U>
Available on crate feature canvas only.
pub fn select_vector<T, U>( self, a: Vector3D<T, U>, b: Vector3D<T, U>, ) -> Vector3D<T, U>
canvas only.Returns vector, each component of which or from a, or from b depending on truly value
of corresponding vector component. true selects value from a and false from b.
pub fn select_size<T, U>(self, a: Size3D<T, U>, b: Size3D<T, U>) -> Size3D<T, U>
Available on crate feature canvas only.
pub fn select_size<T, U>(self, a: Size3D<T, U>, b: Size3D<T, U>) -> Size3D<T, U>
canvas only.Returns size, each component of which or from a, or from b depending on truly value
of corresponding vector component. true selects value from a and false from b.
pub fn xy(self) -> BoolVector2D
Available on crate feature canvas only.
pub fn xy(self) -> BoolVector2D
canvas only.Returns a 2d vector using this vector’s x and y coordinates.
pub fn xz(self) -> BoolVector2D
Available on crate feature canvas only.
pub fn xz(self) -> BoolVector2D
canvas only.Returns a 2d vector using this vector’s x and z coordinates.
pub fn yz(self) -> BoolVector2D
Available on crate feature canvas only.
pub fn yz(self) -> BoolVector2D
canvas only.Returns a 2d vector using this vector’s y and z coordinates.