Struct Box2D
#[repr(C)]pub struct Box2D<T, U> {
pub min: Point2D<T, U>,
pub max: Point2D<T, U>,
}geometry only.Expand description
A 2d axis aligned rectangle represented by its minimum and maximum coordinates.
Representation
This struct is similar to Rect, but stores rectangle as two endpoints
instead of origin point and size. Such representation has several advantages over
Rect representation:
- Several operations are more efficient with
Box2D, includingintersection,union, and point-in-rect. - The representation is less susceptible to overflow. With
Rect, computation of second point can overflow for a large range of values of origin and size. However, withBox2D, computation ofsizecannot overflow if the coordinates are signed and the resulting size is unsigned.
A known disadvantage of Box2D is that translating the rectangle requires translating
both points, whereas translating Rect only requires translating one point.
Empty box
A box is considered empty (see is_empty) if any of the following is true:
- it’s area is empty,
- it’s area is negative (
min.x > max.xormin.y > max.y), - it contains NaNs.
Fields
min: Point2D<T, U>max: Point2D<T, U>Implementations
impl<T, U> Box2D<T, U>
impl<T, U> Box2D<T, U>
impl<T, U> Box2D<T, U>where
T: PartialOrd,
impl<T, U> Box2D<T, U>where
T: PartialOrd,
pub fn is_negative(&self) -> bool
Available on crate feature canvas only.
pub fn is_negative(&self) -> bool
canvas only.Returns true if the box has a negative area.
The common interpretation for a negative box is to consider it empty. It can be obtained by calculating the intersection of two boxes that do not intersect.
pub fn is_empty(&self) -> bool
Available on crate feature canvas only.
pub fn is_empty(&self) -> bool
canvas only.Returns true if the size is zero, negative or NaN.
pub fn intersects(&self, other: &Box2D<T, U>) -> bool
Available on crate feature canvas only.
pub fn intersects(&self, other: &Box2D<T, U>) -> bool
canvas only.Returns true if the two boxes intersect.
pub fn contains(&self, p: Point2D<T, U>) -> bool
Available on crate feature canvas only.
pub fn contains(&self, p: Point2D<T, U>) -> bool
canvas only.Returns true if this Box2D contains the point p.
Points on the top and left edges are inside the box, whereas
points on the bottom and right edges are outside the box.
See Box2D::contains_inclusive for a variant that also includes those
latter points.
Examples
use euclid::default::{Box2D, Point2D};
let rect = Box2D::new(Point2D::origin(), Point2D::new(2, 2));
assert!(rect.contains(Point2D::new(1, 1)));
assert!(rect.contains(Point2D::new(0, 1))); // left edge
assert!(rect.contains(Point2D::new(1, 0))); // top edge
assert!(rect.contains(Point2D::origin()));
assert!(!rect.contains(Point2D::new(2, 1))); // right edge
assert!(!rect.contains(Point2D::new(1, 2))); // bottom edge
assert!(!rect.contains(Point2D::new(2, 2)));pub fn contains_inclusive(&self, p: Point2D<T, U>) -> bool
Available on crate feature canvas only.
pub fn contains_inclusive(&self, p: Point2D<T, U>) -> bool
canvas only.Returns true if this box contains the point p.
This is like Box2D::contains, but points on the bottom and right
edges are also inside the box.
Examples
use euclid::default::{Box2D, Point2D};
let rect = Box2D::new(Point2D::origin(), Point2D::new(2, 2));
assert!(rect.contains_inclusive(Point2D::new(1, 1)));
assert!(rect.contains_inclusive(Point2D::new(0, 1))); // left edge
assert!(rect.contains_inclusive(Point2D::new(1, 0))); // top edge
assert!(rect.contains_inclusive(Point2D::origin()));
assert!(rect.contains_inclusive(Point2D::new(2, 1))); // right edge
assert!(rect.contains_inclusive(Point2D::new(1, 2))); // bottom edge
assert!(rect.contains_inclusive(Point2D::new(2, 2)));pub fn contains_box(&self, other: &Box2D<T, U>) -> bool
Available on crate feature canvas only.
pub fn contains_box(&self, other: &Box2D<T, U>) -> bool
canvas only.Returns true if this box contains the interior of the other box. Always
returns true if other is empty, and always returns false if other is
nonempty but this box is empty.
impl<T, U> Box2D<T, U>where
T: Copy + PartialOrd,
impl<T, U> Box2D<T, U>where
T: Copy + PartialOrd,
pub fn to_non_empty(&self) -> Option<Box2D<T, U>>
canvas only.pub fn intersection(&self, other: &Box2D<T, U>) -> Option<Box2D<T, U>>
Available on crate feature canvas only.
pub fn intersection(&self, other: &Box2D<T, U>) -> Option<Box2D<T, U>>
canvas only.Computes the intersection of two boxes, returning None if the boxes do not intersect.
pub fn intersection_unchecked(&self, other: &Box2D<T, U>) -> Box2D<T, U>
Available on crate feature canvas only.
pub fn intersection_unchecked(&self, other: &Box2D<T, U>) -> Box2D<T, U>
canvas only.Computes the intersection of two boxes without check whether they do intersect.
The result is a negative box if the boxes do not intersect.
This can be useful for computing the intersection of more than two boxes, as
it is possible to chain multiple intersection_unchecked calls and check for
empty/negative result at the end.
impl<T, U> Box2D<T, U>
impl<T, U> Box2D<T, U>
pub fn size(&self) -> Size2D<T, U>
canvas only.pub fn set_size(&mut self, size: Size2D<T, U>)
Available on crate feature canvas only.
pub fn set_size(&mut self, size: Size2D<T, U>)
canvas only.Change the size of the box by adjusting the max endpoint without modifying the min endpoint.
pub fn width(&self) -> T
canvas only.pub fn height(&self) -> T
canvas only.pub fn to_rect(&self) -> Rect<T, U>
canvas only.impl<T, U> Box2D<T, U>
impl<T, U> Box2D<T, U>
pub fn inflate(&self, width: T, height: T) -> Box2D<T, U>
Available on crate feature canvas only.
pub fn inflate(&self, width: T, height: T) -> Box2D<T, U>
canvas only.Inflates the box by the specified sizes on each side respectively.
pub fn inner_box(&self, offsets: SideOffsets2D<T, U>) -> Box2D<T, U>
Available on crate feature canvas only.
pub fn inner_box(&self, offsets: SideOffsets2D<T, U>) -> Box2D<T, U>
canvas only.Calculate the size and position of an inner box.
Subtracts the side offsets from all sides. The horizontal, vertical and applicate offsets must not be larger than the original side length.
pub fn outer_box(&self, offsets: SideOffsets2D<T, U>) -> Box2D<T, U>
Available on crate feature canvas only.
pub fn outer_box(&self, offsets: SideOffsets2D<T, U>) -> Box2D<T, U>
canvas only.Calculate the b and position of an outer box.
Add the offsets to all sides. The expanded box is returned.
impl<T, U> Box2D<T, U>
impl<T, U> Box2D<T, U>
pub fn from_points<I>(points: I) -> Box2D<T, U>
Available on crate feature canvas only.
pub fn from_points<I>(points: I) -> Box2D<T, U>
canvas only.Returns the smallest box enclosing all of the provided points.
The top/bottom/left/right-most points are exactly on the box’s edges.
Since Box2D::contains excludes points that are on the right-most and
bottom-most edges, not all points passed to Box2D::from_points are
contained in the returned Box2D when probed with Box2D::contains, but
are when probed with Box2D::contains_inclusive.
For example:
use euclid::default::{Point2D, Box2D};
let a = Point2D::origin();
let b = Point2D::new(1, 2);
let rect = Box2D::from_points([a, b]);
assert_eq!(rect.width(), 1);
assert_eq!(rect.height(), 2);
assert!(rect.contains(a));
assert!(!rect.contains(b));
assert!(rect.contains_inclusive(b));In particular, calling Box2D::from_points with a single point
results in an empty Box2D:
use euclid::default::{Point2D, Box2D};
let a = Point2D::new(1, 0);
let rect = Box2D::from_points([a]);
assert!(rect.is_empty());
assert!(!rect.contains(a));
assert!(rect.contains_inclusive(a));The Box2D enclosing no points is also empty:
use euclid::default::{Box2D, Point2D};
let rect = Box2D::from_points(std::iter::empty::<Point2D<i32>>());
assert!(rect.is_empty());impl<T, U> Box2D<T, U>where
T: Copy,
impl<T, U> Box2D<T, U>where
T: Copy,
pub fn x_range(&self) -> Range<T>
canvas only.pub fn y_range(&self) -> Range<T>
canvas only.pub fn to_untyped(&self) -> Box2D<T, UnknownUnit>
Available on crate feature canvas only.
pub fn to_untyped(&self) -> Box2D<T, UnknownUnit>
canvas only.Drop the units, preserving only the numeric value.
pub fn from_untyped(c: &Box2D<T, UnknownUnit>) -> Box2D<T, U>
Available on crate feature canvas only.
pub fn from_untyped(c: &Box2D<T, UnknownUnit>) -> Box2D<T, U>
canvas only.Tag a unitless value with units.
pub fn scale<S>(&self, x: S, y: S) -> Box2D<T, U>
canvas only.impl<T, U> Box2D<T, U>
impl<T, U> Box2D<T, U>
pub fn cast<NewT>(&self) -> Box2D<NewT, U>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn cast<NewT>(&self) -> Box2D<NewT, U>where
NewT: NumCast,
canvas only.Cast from one numeric representation to another, preserving the units.
When casting from floating point to integer coordinates, the decimals are truncated
as one would expect from a simple cast, but this behavior does not always make sense
geometrically. Consider using round, round_in or round_out before casting.
pub fn try_cast<NewT>(&self) -> Option<Box2D<NewT, U>>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn try_cast<NewT>(&self) -> Option<Box2D<NewT, U>>where
NewT: NumCast,
canvas only.Fallible cast from one numeric representation to another, preserving the units.
When casting from floating point to integer coordinates, the decimals are truncated
as one would expect from a simple cast, but this behavior does not always make sense
geometrically. Consider using round, round_in or round_out before casting.
pub fn to_usize(&self) -> Box2D<usize, U>
Available on crate feature canvas only.
pub fn to_usize(&self) -> Box2D<usize, U>
canvas only.Cast into an usize box, truncating decimals if any.
When casting from floating point boxes, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
pub fn to_u32(&self) -> Box2D<u32, U>
Available on crate feature canvas only.
pub fn to_u32(&self) -> Box2D<u32, U>
canvas only.Cast into an u32 box, truncating decimals if any.
When casting from floating point boxes, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
pub fn to_i32(&self) -> Box2D<i32, U>
Available on crate feature canvas only.
pub fn to_i32(&self) -> Box2D<i32, U>
canvas only.Cast into an i32 box, truncating decimals if any.
When casting from floating point boxes, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
pub fn to_i64(&self) -> Box2D<i64, U>
Available on crate feature canvas only.
pub fn to_i64(&self) -> Box2D<i64, U>
canvas only.Cast into an i64 box, truncating decimals if any.
When casting from floating point boxes, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
impl<T, U> Box2D<T, U>where
T: Round,
impl<T, U> Box2D<T, U>where
T: Round,
pub fn round(&self) -> Box2D<T, U>
Available on crate feature canvas only.
pub fn round(&self) -> Box2D<T, U>
canvas only.Return a box with edges rounded to integer coordinates, such that the returned box has the same set of pixel centers as the original one. Values equal to 0.5 round up. Suitable for most places where integral device coordinates are needed, but note that any translation should be applied first to avoid pixel rounding errors. Note that this is not rounding to nearest integer if the values are negative. They are always rounding as floor(n + 0.5).