Struct Box3D
#[repr(C)]pub struct Box3D<T, U> {
pub min: Point3D<T, U>,
pub max: Point3D<T, U>,
}geometry only.Expand description
An axis aligned 3D box represented by its minimum and maximum coordinates.
Fields
min: Point3D<T, U>max: Point3D<T, U>Implementations
impl<T, U> Box3D<T, U>
impl<T, U> Box3D<T, U>
impl<T, U> Box3D<T, U>where
T: PartialOrd,
impl<T, U> Box3D<T, U>where
T: PartialOrd,
pub fn is_negative(&self) -> bool
Available on crate feature canvas only.
pub fn is_negative(&self) -> bool
canvas only.Returns true if the box has a negative volume.
The common interpretation for a negative box is to consider it empty. It can be obtained by calculating the intersection of two boxes that do not intersect.
pub fn is_empty(&self) -> bool
Available on crate feature canvas only.
pub fn is_empty(&self) -> bool
canvas only.Returns true if the size is zero, negative or NaN.
pub fn intersects(&self, other: &Box3D<T, U>) -> bool
canvas only.pub fn contains(&self, other: Point3D<T, U>) -> bool
Available on crate feature canvas only.
pub fn contains(&self, other: Point3D<T, U>) -> bool
canvas only.Returns true if this Box3D contains the point p.
Points on the front, left, and top faces are inside the box, whereas
points on the back, right, and bottom faces are outside the box.
See Box3D::contains_inclusive for a variant that also includes those
latter points.
Examples
use euclid::default::{Box3D, Point3D};
let cube = Box3D::new(Point3D::origin(), Point3D::new(2, 2, 2));
assert!(cube.contains(Point3D::new(1, 1, 1)));
assert!(cube.contains(Point3D::new(0, 1, 1))); // front face
assert!(cube.contains(Point3D::new(1, 0, 1))); // left face
assert!(cube.contains(Point3D::new(1, 1, 0))); // top face
assert!(cube.contains(Point3D::new(0, 0, 0)));
assert!(!cube.contains(Point3D::new(2, 1, 1))); // back face
assert!(!cube.contains(Point3D::new(1, 2, 1))); // right face
assert!(!cube.contains(Point3D::new(1, 1, 2))); // bottom face
assert!(!cube.contains(Point3D::new(2, 2, 2)));pub fn contains_inclusive(&self, other: Point3D<T, U>) -> bool
Available on crate feature canvas only.
pub fn contains_inclusive(&self, other: Point3D<T, U>) -> bool
canvas only.Returns true if this Box3D contains the point p.
This is like Box3D::contains, but points on the back, right,
and bottom faces are also inside the box.
Examples
use euclid::default::{Box3D, Point3D};
let cube = Box3D::new(Point3D::origin(), Point3D::new(2, 2, 2));
assert!(cube.contains_inclusive(Point3D::new(1, 1, 1)));
assert!(cube.contains_inclusive(Point3D::new(0, 1, 1))); // front face
assert!(cube.contains_inclusive(Point3D::new(1, 0, 1))); // left face
assert!(cube.contains_inclusive(Point3D::new(1, 1, 0))); // top face
assert!(cube.contains_inclusive(Point3D::new(0, 0, 0))); // front-left-top corner
assert!(cube.contains_inclusive(Point3D::new(2, 1, 1))); // back face
assert!(cube.contains_inclusive(Point3D::new(1, 2, 1))); // right face
assert!(cube.contains_inclusive(Point3D::new(1, 1, 2))); // bottom face
assert!(cube.contains_inclusive(Point3D::new(2, 2, 2))); // back-right-bottom cornerpub fn contains_box(&self, other: &Box3D<T, U>) -> bool
Available on crate feature canvas only.
pub fn contains_box(&self, other: &Box3D<T, U>) -> bool
canvas only.Returns true if this box3d contains the interior of the other box3d. Always
returns true if other is empty, and always returns false if other is
nonempty but this box3d is empty.
impl<T, U> Box3D<T, U>where
T: Copy + PartialOrd,
impl<T, U> Box3D<T, U>where
T: Copy + PartialOrd,
pub fn to_non_empty(&self) -> Option<Box3D<T, U>>
canvas only.pub fn intersection(&self, other: &Box3D<T, U>) -> Option<Box3D<T, U>>
canvas only.pub fn intersection_unchecked(&self, other: &Box3D<T, U>) -> Box3D<T, U>
canvas only.impl<T, U> Box3D<T, U>
impl<T, U> Box3D<T, U>
pub fn from_points<I>(points: I) -> Box3D<T, U>
Available on crate feature canvas only.
pub fn from_points<I>(points: I) -> Box3D<T, U>
canvas only.Returns the smallest box enclosing all of the provided points.
The top/bottom/left/right/front/back-most points are exactly on the box’s edges.
Since Box3D::contains excludes points that are on the right/bottom/back-most
faces, not all points passed to Box3D::from_points are
contained in the returned Box3D when probed with Box3D::contains, but
are when probed with Box3D::contains_inclusive.
For example:
use euclid::default::{Point3D, Box3D};
let a = Point3D::origin();
let b = Point3D::new(1, 2, 3);
let box3 = Box3D::from_points([a, b]);
assert_eq!(box3.width(), 1);
assert_eq!(box3.height(), 2);
assert_eq!(box3.depth(), 3);
assert!(box3.contains(a));
assert!(!box3.contains(b));
assert!(box3.contains_inclusive(b));In particular, calling Box3D::from_points with a single point
results in an empty Box3D:
use euclid::default::{Point3D, Box3D};
let a = Point3D::new(1, 0, 1);
let box3 = Box3D::from_points([a]);
assert!(box3.is_empty());
assert!(!box3.contains(a));
assert!(box3.contains_inclusive(a));The Box3D enclosing no points is also empty:
use euclid::default::{Box3D, Point3D};
let box3 = Box3D::from_points(std::iter::empty::<Point3D<i32>>());
assert!(box3.is_empty());impl<T, U> Box3D<T, U>where
T: Copy,
impl<T, U> Box3D<T, U>where
T: Copy,
pub fn x_range(&self) -> Range<T>
canvas only.pub fn y_range(&self) -> Range<T>
canvas only.pub fn z_range(&self) -> Range<T>
canvas only.pub fn to_untyped(&self) -> Box3D<T, UnknownUnit>
Available on crate feature canvas only.
pub fn to_untyped(&self) -> Box3D<T, UnknownUnit>
canvas only.Drop the units, preserving only the numeric value.
pub fn from_untyped(c: &Box3D<T, UnknownUnit>) -> Box3D<T, U>
Available on crate feature canvas only.
pub fn from_untyped(c: &Box3D<T, UnknownUnit>) -> Box3D<T, U>
canvas only.Tag a unitless value with units.
pub fn scale<S>(&self, x: S, y: S, z: S) -> Box3D<T, U>
canvas only.impl<T, U> Box3D<T, U>
impl<T, U> Box3D<T, U>
pub fn cast<NewT>(&self) -> Box3D<NewT, U>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn cast<NewT>(&self) -> Box3D<NewT, U>where
NewT: NumCast,
canvas only.Cast from one numeric representation to another, preserving the units.
When casting from floating point to integer coordinates, the decimals are truncated
as one would expect from a simple cast, but this behavior does not always make sense
geometrically. Consider using round, round_in or round_out before casting.
pub fn try_cast<NewT>(&self) -> Option<Box3D<NewT, U>>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn try_cast<NewT>(&self) -> Option<Box3D<NewT, U>>where
NewT: NumCast,
canvas only.Fallible cast from one numeric representation to another, preserving the units.
When casting from floating point to integer coordinates, the decimals are truncated
as one would expect from a simple cast, but this behavior does not always make sense
geometrically. Consider using round, round_in or round_out before casting.
pub fn to_usize(&self) -> Box3D<usize, U>
Available on crate feature canvas only.
pub fn to_usize(&self) -> Box3D<usize, U>
canvas only.Cast into an usize box3d, truncating decimals if any.
When casting from floating point cuboids, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
pub fn to_u32(&self) -> Box3D<u32, U>
Available on crate feature canvas only.
pub fn to_u32(&self) -> Box3D<u32, U>
canvas only.Cast into an u32 box3d, truncating decimals if any.
When casting from floating point cuboids, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
pub fn to_i32(&self) -> Box3D<i32, U>
Available on crate feature canvas only.
pub fn to_i32(&self) -> Box3D<i32, U>
canvas only.Cast into an i32 box3d, truncating decimals if any.
When casting from floating point cuboids, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
pub fn to_i64(&self) -> Box3D<i64, U>
Available on crate feature canvas only.
pub fn to_i64(&self) -> Box3D<i64, U>
canvas only.Cast into an i64 box3d, truncating decimals if any.
When casting from floating point cuboids, it is worth considering whether
to round(), round_in() or round_out() before the cast in order to
obtain the desired conversion behavior.
impl<T, U> Box3D<T, U>where
T: Round,
impl<T, U> Box3D<T, U>where
T: Round,
pub fn round(&self) -> Box3D<T, U>
Available on crate feature canvas only.
pub fn round(&self) -> Box3D<T, U>
canvas only.Return a box3d with edges rounded to integer coordinates, such that the returned box3d has the same set of pixel centers as the original one. Values equal to 0.5 round up. Suitable for most places where integral device coordinates are needed, but note that any translation should be applied first to avoid pixel rounding errors. Note that this is not rounding to nearest integer if the values are negative. They are always rounding as floor(n + 0.5).