Point3D

Struct Point3D 

pub struct Point3D<T, U> {
    pub x: T,
    pub y: T,
    pub z: T,
    /* private fields */
}
Available on crate feature geometry only.
A 3d Point tagged with a unit.

Fields

x: Ty: Tz: T

Implementations



impl<T, U> Point3D<T, U>

pub fn origin() -> Point3D<T, U>
where T: Zero,

Available on crate feature canvas only.

Constructor, setting all components to zero.

pub fn zero() -> Point3D<T, U>
where T: Zero,

Available on crate feature canvas only.

The same as Point3D::origin.

pub const fn new(x: T, y: T, z: T) -> Point3D<T, U>

Available on crate feature canvas only.

Constructor taking scalar values directly.

pub fn from_lengths( x: Length<T, U>, y: Length<T, U>, z: Length<T, U>, ) -> Point3D<T, U>

Available on crate feature canvas only.

Constructor taking properly Lengths instead of scalar values.

pub fn splat(v: T) -> Point3D<T, U>
where T: Clone,

Available on crate feature canvas only.

Constructor setting all components to the same value.

pub fn from_untyped(p: Point3D<T, UnknownUnit>) -> Point3D<T, U>

Available on crate feature canvas only.

Tag a unitless value with units.

pub fn map<V, F>(self, f: F) -> Point3D<V, U>
where F: FnMut(T) -> V,

Available on crate feature canvas only.

Apply the function f to each component of this point.

Example

This may be used to perform unusual arithmetic which is not already offered as methods.

use euclid::default::Point3D;

let p = Point3D::<u32>::new(5, 11, 15);
assert_eq!(p.map(|coord| coord.saturating_sub(10)), Point3D::new(0, 1, 5));

pub fn zip<V, F>(self, rhs: Point3D<T, U>, f: F) -> Vector3D<V, U>
where F: FnMut(T, T) -> V,

Available on crate feature canvas only.

Apply the function f to each pair of components of this point and rhs.

Example

This may be used to perform unusual arithmetic which is not already offered as methods.

use euclid::{default::{Point3D, Vector3D}, point2};

let a: Point3D<u32> = Point3D::new(50, 200, 400);
let b: Point3D<u32> = Point3D::new(100, 100, 150);
assert_eq!(a.zip(b, u32::saturating_sub), Vector3D::new(0, 100, 250));


impl<T, U> Point3D<T, U>
where T: Copy,

pub fn to_vector(self) -> Vector3D<T, U>

Available on crate feature canvas only.

Cast this point into a vector.

Equivalent to subtracting the origin to this point.

pub fn xy(self) -> Point2D<T, U>

Available on crate feature canvas only.

Returns a 2d point using this point’s x and y coordinates

pub fn xz(self) -> Point2D<T, U>

Available on crate feature canvas only.

Returns a 2d point using this point’s x and z coordinates

pub fn yz(self) -> Point2D<T, U>

Available on crate feature canvas only.

Returns a 2d point using this point’s x and z coordinates

pub fn to_array(self) -> [T; 3]

Available on crate feature canvas only.

Cast into an array with x, y and z.

Example
enum Mm {}

let point: Point3D<_, Mm> = point3(1, -8, 0);

assert_eq!(point.to_array(), [1, -8, 0]);

pub fn to_array_4d(self) -> [T; 4]
where T: One,

Available on crate feature canvas only.

pub fn to_tuple(self) -> (T, T, T)

Available on crate feature canvas only.

Cast into a tuple with x, y and z.

Example
enum Mm {}

let point: Point3D<_, Mm> = point3(1, -8, 0);

assert_eq!(point.to_tuple(), (1, -8, 0));

pub fn to_tuple_4d(self) -> (T, T, T, T)
where T: One,

Available on crate feature canvas only.

pub fn to_untyped(self) -> Point3D<T, UnknownUnit>

Available on crate feature canvas only.

Drop the units, preserving only the numeric value.

Example
enum Mm {}

let point: Point3D<_, Mm> = point3(1, -8, 0);

assert_eq!(point.x, point.to_untyped().x);
assert_eq!(point.y, point.to_untyped().y);
assert_eq!(point.z, point.to_untyped().z);

pub fn cast_unit<V>(self) -> Point3D<T, V>

Available on crate feature canvas only.

Cast the unit, preserving the numeric value.

Example
enum Mm {}
enum Cm {}

let point: Point3D<_, Mm> = point3(1, -8, 0);

assert_eq!(point.x, point.cast_unit::<Cm>().x);
assert_eq!(point.y, point.cast_unit::<Cm>().y);
assert_eq!(point.z, point.cast_unit::<Cm>().z);

pub fn to_2d(self) -> Point2D<T, U>

Available on crate feature canvas only.

Convert into a 2d point.

pub fn round(self) -> Point3D<T, U>
where T: Round,

Available on crate feature canvas only.

Rounds each component to the nearest integer value.

This behavior is preserved for negative values (unlike the basic cast).

enum Mm {}

assert_eq!(point3::<_, Mm>(-0.1, -0.8, 0.4).round(), point3::<_, Mm>(0.0, -1.0, 0.0))

pub fn ceil(self) -> Point3D<T, U>
where T: Ceil,

Available on crate feature canvas only.

Rounds each component to the smallest integer equal or greater than the original value.

This behavior is preserved for negative values (unlike the basic cast).

enum Mm {}

assert_eq!(point3::<_, Mm>(-0.1, -0.8, 0.4).ceil(), point3::<_, Mm>(0.0, 0.0, 1.0))

pub fn floor(self) -> Point3D<T, U>
where T: Floor,

Available on crate feature canvas only.

Rounds each component to the biggest integer equal or lower than the original value.

This behavior is preserved for negative values (unlike the basic cast).

enum Mm {}

assert_eq!(point3::<_, Mm>(-0.1, -0.8, 0.4).floor(), point3::<_, Mm>(-1.0, -1.0, 0.0))

pub fn lerp(self, other: Point3D<T, U>, t: T) -> Point3D<T, U>
where T: One + Sub<Output = T> + Mul<Output = T> + Add<Output = T>,

Available on crate feature canvas only.

Linearly interpolate between this point and another point.

Example
use euclid::point3;
use euclid::default::Point3D;

let from: Point3D<_> = point3(0.0, 10.0, -1.0);
let to:  Point3D<_> = point3(8.0, -4.0,  0.0);

assert_eq!(from.lerp(to, -1.0), point3(-8.0,  24.0, -2.0));
assert_eq!(from.lerp(to,  0.0), point3( 0.0,  10.0, -1.0));
assert_eq!(from.lerp(to,  0.5), point3( 4.0,   3.0, -0.5));
assert_eq!(from.lerp(to,  1.0), point3( 8.0,  -4.0,  0.0));
assert_eq!(from.lerp(to,  2.0), point3(16.0, -18.0,  1.0));


impl<T, U> Point3D<T, U>
where T: PartialOrd,

pub fn min(self, other: Point3D<T, U>) -> Point3D<T, U>

Available on crate feature canvas only.

pub fn max(self, other: Point3D<T, U>) -> Point3D<T, U>

Available on crate feature canvas only.

pub fn clamp(self, start: Point3D<T, U>, end: Point3D<T, U>) -> Point3D<T, U>
where T: Copy,

Available on crate feature canvas only.

Returns the point each component of which clamped by corresponding components of start and end.

Shortcut for self.max(start).min(end).



impl<T, U> Point3D<T, U>
where T: NumCast + Copy,

pub fn cast<NewT>(self) -> Point3D<NewT, U>
where NewT: NumCast,

Available on crate feature canvas only.

Cast from one numeric representation to another, preserving the units.

When casting from floating point to integer coordinates, the decimals are truncated as one would expect from a simple cast, but this behavior does not always make sense geometrically. Consider using round(), ceil() or floor() before casting.

pub fn try_cast<NewT>(self) -> Option<Point3D<NewT, U>>
where NewT: NumCast,

Available on crate feature canvas only.

Fallible cast from one numeric representation to another, preserving the units.

When casting from floating point to integer coordinates, the decimals are truncated as one would expect from a simple cast, but this behavior does not always make sense geometrically. Consider using round(), ceil() or floor() before casting.

pub fn to_f32(self) -> Point3D<f32, U>

Available on crate feature canvas only.

Cast into an f32 point.

pub fn to_f64(self) -> Point3D<f64, U>

Available on crate feature canvas only.

Cast into an f64 point.

pub fn to_usize(self) -> Point3D<usize, U>

Available on crate feature canvas only.

Cast into an usize point, truncating decimals if any.

When casting from floating point points, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.

pub fn to_u32(self) -> Point3D<u32, U>

Available on crate feature canvas only.

Cast into an u32 point, truncating decimals if any.

When casting from floating point points, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.

pub fn to_i32(self) -> Point3D<i32, U>

Available on crate feature canvas only.

Cast into an i32 point, truncating decimals if any.

When casting from floating point points, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.

pub fn to_i64(self) -> Point3D<i64, U>

Available on crate feature canvas only.

Cast into an i64 point, truncating decimals if any.

When casting from floating point points, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.



impl<T, U> Point3D<T, U>
where T: Copy + Add<Output = T>,

pub fn add_size(self, other: Size3D<T, U>) -> Point3D<T, U>

Available on crate feature canvas only.


impl<T, U> Point3D<T, U>
where T: Euclid,

pub fn rem_euclid(&self, other: &Size3D<T, U>) -> Point3D<T, U>

Available on crate feature canvas only.

Calculates the least nonnegative remainder of self (mod other).

Example
use euclid::point3;
use euclid::default::{Point3D, Size3D};

let p = Point3D::new(7.0, -7.0, 0.0);
let s = Size3D::new(4.0, -4.0, 12.0);

assert_eq!(p.rem_euclid(&s), point3(3.0, 1.0, 0.0));
assert_eq!((-p).rem_euclid(&s), point3(1.0, 3.0, 0.0));
assert_eq!(p.rem_euclid(&-s), point3(3.0, 1.0, 0.0));

pub fn div_euclid(&self, other: &Size3D<T, U>) -> Point3D<T, U>

Available on crate feature canvas only.

Calculates Euclidean division, the matching method for rem_euclid.

Example
use euclid::point3;
use euclid::default::{Point3D, Size3D};

let p = Point3D::new(7.0, -7.0, 0.0);
let s = Size3D::new(4.0, -4.0, 12.0);

assert_eq!(p.div_euclid(&s), point3(1.0, 2.0, 0.0));
assert_eq!((-p).div_euclid(&s), point3(-2.0, -1.0, 0.0));
assert_eq!(p.div_euclid(&-s), point3(-1.0, -2.0, 0.0));


impl<T, U> Point3D<T, U>
where T: Float,

pub fn is_finite(self) -> bool

Available on crate feature canvas only.

Returns true if all members are finite.



impl<T, U> Point3D<T, U>
where T: Real<Output = T> + Sub,

pub fn distance_to(self, other: Point3D<T, U>) -> T

Available on crate feature canvas only.