Rect

Struct Rect 

#[repr(C)]
pub struct Rect<T, U> { pub origin: Point2D<T, U>, pub size: Size2D<T, U>, }
Available on crate feature geometry only.
A 2d Rectangle optionally tagged with a unit.

Representation

Rect is represented by an origin point and a size.

See Box2D for a rectangle represented by two endpoints.

Empty rectangle

A rectangle is considered empty (see is_empty) if any of the following is true:

  • it’s area is empty,
  • it’s area is negative (size.x < 0 or size.y < 0),
  • it contains NaNs.

Fields

origin: Point2D<T, U>size: Size2D<T, U>

Implementations



impl<T, U> Rect<T, U>

pub const fn new(origin: Point2D<T, U>, size: Size2D<T, U>) -> Rect<T, U>

Available on crate feature canvas only.

Constructor.



impl<T, U> Rect<T, U>
where T: Zero,

pub fn zero() -> Rect<T, U>

Available on crate feature canvas only.

Constructor, setting all sides to zero.

pub fn from_size(size: Size2D<T, U>) -> Rect<T, U>

Available on crate feature canvas only.

Creates a rect of the given size, at offset zero.



impl<T, U> Rect<T, U>
where T: Copy + Add<Output = T>,

pub fn min(&self) -> Point2D<T, U>

Available on crate feature canvas only.

pub fn max(&self) -> Point2D<T, U>

Available on crate feature canvas only.

pub fn max_x(&self) -> T

Available on crate feature canvas only.

pub fn min_x(&self) -> T

Available on crate feature canvas only.

pub fn max_y(&self) -> T

Available on crate feature canvas only.

pub fn min_y(&self) -> T

Available on crate feature canvas only.

pub fn width(&self) -> T

Available on crate feature canvas only.

pub fn height(&self) -> T

Available on crate feature canvas only.

pub fn x_range(&self) -> Range<T>

Available on crate feature canvas only.

pub fn y_range(&self) -> Range<T>

Available on crate feature canvas only.

pub fn translate(&self, by: Vector2D<T, U>) -> Rect<T, U>

Available on crate feature canvas only.

Returns the same rectangle, translated by a vector.

pub fn to_box2d(&self) -> Box2D<T, U>

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + PartialOrd + Add<Output = T>,

pub fn contains(&self, p: Point2D<T, U>) -> bool

Available on crate feature canvas only.

Returns true if this rectangle contains the point. Points are considered in the rectangle if they are on the left or top edge, but outside if they are on the right or bottom edge.

pub fn intersects(&self, other: &Rect<T, U>) -> bool

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + PartialOrd + Add<Output = T> + Sub<Output = T>,

pub fn intersection(&self, other: &Rect<T, U>) -> Option<Rect<T, U>>

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + Add<Output = T> + Sub<Output = T>,

pub fn inflate(&self, width: T, height: T) -> Rect<T, U>

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + Zero + PartialOrd + Add<Output = T>,

pub fn contains_rect(&self, rect: &Rect<T, U>) -> bool

Available on crate feature canvas only.

Returns true if this rectangle contains the interior of rect. Always returns true if rect is empty, and always returns false if rect is nonempty but this rectangle is empty.



impl<T, U> Rect<T, U>
where T: Copy + Zero + PartialOrd + Add<Output = T> + Sub<Output = T>,

pub fn inner_rect(&self, offsets: SideOffsets2D<T, U>) -> Rect<T, U>

Available on crate feature canvas only.

Calculate the size and position of an inner rectangle.

Subtracts the side offsets from all sides. The horizontal and vertical offsets must not be larger than the original side length. This method assumes y oriented downward.



impl<T, U> Rect<T, U>
where T: Copy + Add<Output = T> + Sub<Output = T>,

pub fn outer_rect(&self, offsets: SideOffsets2D<T, U>) -> Rect<T, U>

Available on crate feature canvas only.

Calculate the size and position of an outer rectangle.

Add the offsets to all sides. The expanded rectangle is returned. This method assumes y oriented downward.



impl<T, U> Rect<T, U>
where T: Copy + Zero + PartialOrd + Sub<Output = T>,

pub fn from_points<I>(points: I) -> Rect<T, U>
where I: IntoIterator, <I as IntoIterator>::Item: Borrow<Point2D<T, U>>,

Available on crate feature canvas only.

Returns the smallest rectangle defined by the top/bottom/left/right-most points provided as parameter.

Note: This function has a behavior that can be surprising because the right-most and bottom-most points are exactly on the edge of the rectangle while the Rect::contains function is has exclusive semantic on these edges. This means that the right-most and bottom-most points provided to Rect::from_points will count as not contained by the rect. This behavior may change in the future.

See Box2D::from_points for more details.



impl<T, U> Rect<T, U>
where T: One + Add<Output = T> + Sub<Output = T> + Mul<Output = T> + Copy,

pub fn lerp(&self, other: Rect<T, U>, t: T) -> Rect<T, U>

Available on crate feature canvas only.

Linearly interpolate between this rectangle and another rectangle.



impl<T, U> Rect<T, U>
where T: Copy + One + Add<Output = T> + Div<Output = T>,

pub fn center(&self) -> Point2D<T, U>

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + PartialOrd + Add<Output = T> + Sub<Output = T> + Zero,

pub fn union(&self, other: &Rect<T, U>) -> Rect<T, U>

Available on crate feature canvas only.


impl<T, U> Rect<T, U>

pub fn scale<S>(&self, x: S, y: S) -> Rect<T, U>
where S: Copy, T: Copy + Mul<S, Output = T>,

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + Mul<Output = T>,

pub fn area(&self) -> T

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + Zero + PartialOrd,

pub fn is_empty(&self) -> bool

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy + Zero + PartialOrd,

pub fn to_non_empty(&self) -> Option<Rect<T, U>>

Available on crate feature canvas only.


impl<T, U> Rect<T, U>
where T: Copy,

pub fn to_untyped(&self) -> Rect<T, UnknownUnit>

Available on crate feature canvas only.

Drop the units, preserving only the numeric value.

pub fn from_untyped(r: &Rect<T, UnknownUnit>) -> Rect<T, U>

Available on crate feature canvas only.

Tag a unitless value with units.

pub fn cast_unit<V>(&self) -> Rect<T, V>

Available on crate feature canvas only.

Cast the unit



impl<T, U> Rect<T, U>
where T: NumCast + Copy,

pub fn cast<NewT>(&self) -> Rect<NewT, U>
where NewT: NumCast,

Available on crate feature canvas only.

Cast from one numeric representation to another, preserving the units.

When casting from floating point to integer coordinates, the decimals are truncated as one would expect from a simple cast, but this behavior does not always make sense geometrically. Consider using round, round_in or round_out before casting.

pub fn try_cast<NewT>(&self) -> Option<Rect<NewT, U>>
where NewT: NumCast,

Available on crate feature canvas only.

Fallible cast from one numeric representation to another, preserving the units.

When casting from floating point to integer coordinates, the decimals are truncated as one would expect from a simple cast, but this behavior does not always make sense geometrically. Consider using round, round_in or [round_out before casting.

pub fn to_f32(&self) -> Rect<f32, U>

Available on crate feature canvas only.

Cast into an f32 rectangle.

pub fn to_f64(&self) -> Rect<f64, U>

Available on crate feature canvas only.

Cast into an f64 rectangle.

pub fn to_usize(&self) -> Rect<usize, U>

Available on crate feature canvas only.

Cast into an usize rectangle, truncating decimals if any.

When casting from floating point rectangles, it is worth considering whether to round(), round_in() or round_out() before the cast in order to obtain the desired conversion behavior.

pub fn to_u32(&self) -> Rect<u32, U>

Available on crate feature canvas only.

Cast into an u32 rectangle, truncating decimals if any.

When casting from floating point rectangles, it is worth considering whether to round(), round_in() or round_out() before the cast in order to obtain the desired conversion behavior.

pub fn to_u64(&self) -> Rect<u64, U>

Available on crate feature canvas only.

Cast into an u64 rectangle, truncating decimals if any.

When casting from floating point rectangles, it is worth considering whether to round(), round_in() or round_out() before the cast in order to obtain the desired conversion behavior.

pub fn to_i32(&self) -> Rect<i32, U>

Available on crate feature canvas only.

Cast into an i32 rectangle, truncating decimals if any.

When casting from floating point rectangles, it is worth considering whether to round(), round_in() or round_out() before the cast in order to obtain the desired conversion behavior.

pub fn to_i64(&self) -> Rect<i64, U>

Available on crate feature canvas only.

Cast into an i64 rectangle, truncating decimals if any.

When casting from floating point rectangles, it is worth considering whether to round(), round_in() or round_out() before the cast in order to obtain the desired conversion behavior.



impl<T, U> Rect<T, U>
where T: Float,

pub fn is_finite(self) -> bool

Available on crate feature canvas and (crate features std or libm) only.

Returns true if all members are finite.



impl<T, U> Rect<T, U>
where T: Floor + Ceil + Round + Add<Output = T> + Sub<Output = T>,

pub fn round(&self) -> Rect<T, U>

Available on crate feature canvas only.

Return a rectangle with edges rounded to integer coordinates, such that the returned rectangle has the same set of pixel centers as the original one. Edges at offset 0.5 round up. Suitable for most places where integral device coordinates are needed, but note that any translation should be applied first to avoid pixel rounding errors. Note that this is not rounding to nearest integer if the values are negative. They are always rounding as floor(n + 0.5).

Usage notes

Note, that when using with floating-point T types that method can significantly lose precision for large values, so if you need to call this method very often it is better to use Box2D.

pub fn round_in(&self) -> Rect<T, U>

Available on crate feature canvas only.

Return a rectangle with edges rounded to integer coordinates, such that the original rectangle contains the resulting rectangle.

Usage notes

Note, that when using with floating-point T types that method can significantly lose precision for large values, so if you need to call this method very often it is better to use Box2D.

pub fn round_out(&self) -> Rect<T, U>

Available on crate feature canvas only.

Return a rectangle with edges rounded to integer coordinates, such that the original rectangle is contained in the resulting rectangle.

Usage notes

Note, that when using with floating-point T types that method can significantly lose precision for large values, so if you need to call this method very often it is better to use Box2D.