Rotation2D

Struct Rotation2D 

pub struct Rotation2D<T, Src, Dst> {
    pub angle: T,
    /* private fields */
}
Available on crate feature geometry only.
A transform that can represent rotations in 2d, represented as an angle in radians.

Fields

angle: T

Angle in radians

Implementations



impl<T, Src, Dst> Rotation2D<T, Src, Dst>

pub fn new(angle: Angle<T>) -> Rotation2D<T, Src, Dst>

Available on crate feature canvas only.

Creates a rotation from an angle in radians.

pub fn radians(angle: T) -> Rotation2D<T, Src, Dst>

Available on crate feature canvas only.

Creates a rotation from an angle in radians.

pub fn identity() -> Rotation2D<T, Src, Dst>
where T: Zero,

Available on crate feature canvas only.

Creates the identity rotation.



impl<T, Src, Dst> Rotation2D<T, Src, Dst>
where T: Copy,

pub fn cast_unit<Src2, Dst2>(&self) -> Rotation2D<T, Src2, Dst2>

Available on crate feature canvas only.

Cast the unit, preserving the numeric value.

Example
enum Local {}
enum World {}

enum Local2 {}
enum World2 {}

let to_world: Rotation2D<_, Local, World> = Rotation2D::radians(42);

assert_eq!(to_world.angle, to_world.cast_unit::<Local2, World2>().angle);

pub fn to_untyped(&self) -> Rotation2D<T, UnknownUnit, UnknownUnit>

Available on crate feature canvas only.

Drop the units, preserving only the numeric value.

Example
enum Local {}
enum World {}

let to_world: Rotation2D<_, Local, World> = Rotation2D::radians(42);

assert_eq!(to_world.angle, to_world.to_untyped().angle);

pub fn from_untyped( r: &Rotation2D<T, UnknownUnit, UnknownUnit>, ) -> Rotation2D<T, Src, Dst>

Available on crate feature canvas only.

Tag a unitless value with units.

Example
use euclid::UnknownUnit;
enum Local {}
enum World {}

let rot: Rotation2D<_, UnknownUnit, UnknownUnit> = Rotation2D::radians(42);

assert_eq!(rot.angle, Rotation2D::<_, Local, World>::from_untyped(&rot).angle);


impl<T, Src, Dst> Rotation2D<T, Src, Dst>
where T: Copy,

pub fn get_angle(&self) -> Angle<T>

Available on crate feature canvas only.

Returns self.angle as a strongly typed Angle<T>.



impl<T, Src, Dst> Rotation2D<T, Src, Dst>
where T: Real,

pub fn to_3d(&self) -> Rotation3D<T, Src, Dst>

Available on crate feature canvas only.

Creates a 3d rotation (around the z axis) from this 2d rotation.

pub fn inverse(&self) -> Rotation2D<T, Dst, Src>

Available on crate feature canvas only.

Returns the inverse of this rotation.

pub fn then<NewSrc>( &self, other: &Rotation2D<T, NewSrc, Src>, ) -> Rotation2D<T, NewSrc, Dst>

Available on crate feature canvas only.

Returns a rotation representing the other rotation followed by this rotation.

pub fn transform_point(&self, point: Point2D<T, Src>) -> Point2D<T, Dst>

Available on crate feature canvas only.

Returns the given 2d point transformed by this rotation.

The input point must be use the unit Src, and the returned point has the unit Dst.

pub fn transform_vector(&self, vector: Vector2D<T, Src>) -> Vector2D<T, Dst>

Available on crate feature canvas only.

Returns the given 2d vector transformed by this rotation.

The input point must be use the unit Src, and the returned point has the unit Dst.



impl<T, Src, Dst> Rotation2D<T, Src, Dst>
where T: Copy + Add<Output = T> + Sub<Output = T> + Mul<Output = T> + Zero + Trig,

pub fn to_transform(&self) -> Transform2D<T, Src, Dst>

Available on crate feature canvas only.

Returns the matrix representation of this rotation.