Struct Transform3D
pub struct Transform3D<T, Src, Dst> {Show 16 fields
pub m11: T,
pub m12: T,
pub m13: T,
pub m14: T,
pub m21: T,
pub m22: T,
pub m23: T,
pub m24: T,
pub m31: T,
pub m32: T,
pub m33: T,
pub m34: T,
pub m41: T,
pub m42: T,
pub m43: T,
pub m44: T,
/* private fields */
}geometry only.Expand description
A 3d transform stored as a column-major 4 by 4 matrix.
Transforms can be parametrized over the source and destination units, to describe a
transformation from a space to another.
For example, Transform3D<f32, WorldSpace, ScreenSpace>::transform_point3d
takes a Point3D<f32, WorldSpace> and returns a Point3D<f32, ScreenSpace>.
Transforms expose a set of convenience methods for pre- and post-transformations.
Pre-transformations (pre_* methods) correspond to adding an operation that is
applied before the rest of the transformation, while post-transformations (then_*
methods) add an operation that is applied after.
When translating Transform3D into general matrix representations, consider that the
representation follows the column major notation with column vectors.
|x'| | m11 m12 m13 m14 | |x|
|y'| | m21 m22 m23 m24 | |y|
|z'| = | m31 m32 m33 m34 | x |z|
|w | | m41 m42 m43 m44 | |1|The translation terms are m41, m42 and m43.
Fields
m11: Tm12: Tm13: Tm14: Tm21: Tm22: Tm23: Tm24: Tm31: Tm32: Tm33: Tm34: Tm41: Tm42: Tm43: Tm44: TImplementations
impl<T, Src, Dst> Transform3D<T, Src, Dst>
impl<T, Src, Dst> Transform3D<T, Src, Dst>
pub const fn new(
m11: T,
m12: T,
m13: T,
m14: T,
m21: T,
m22: T,
m23: T,
m24: T,
m31: T,
m32: T,
m33: T,
m34: T,
m41: T,
m42: T,
m43: T,
m44: T,
) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub const fn new( m11: T, m12: T, m13: T, m14: T, m21: T, m22: T, m23: T, m24: T, m31: T, m32: T, m33: T, m34: T, m41: T, m42: T, m43: T, m44: T, ) -> Transform3D<T, Src, Dst>
canvas only.Create a transform specifying all of it’s component as a 4 by 4 matrix.
Components are specified following column-major-column-vector matrix notation. For example, the translation terms m41, m42, m43 are the 13rd, 14th and 15th parameters.
use euclid::default::Transform3D;
let tx = 1.0;
let ty = 2.0;
let tz = 3.0;
let translation = Transform3D::new(
1.0, 0.0, 0.0, 0.0,
0.0, 1.0, 0.0, 0.0,
0.0, 0.0, 1.0, 0.0,
tx, ty, tz, 1.0,
);pub fn new_2d(
m11: T,
m12: T,
m21: T,
m22: T,
m41: T,
m42: T,
) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn new_2d( m11: T, m12: T, m21: T, m22: T, m41: T, m42: T, ) -> Transform3D<T, Src, Dst>
canvas only.Create a transform representing a 2d transformation from the components of a 2 by 3 matrix transformation.
Components follow the column-major-column-vector notation (m41 and m42 representing the translation terms).
m11 m12 0 0
m21 m22 0 0
0 0 1 0
m41 m42 0 1impl<T, Src, Dst> Transform3D<T, Src, Dst>where
T: Copy,
impl<T, Src, Dst> Transform3D<T, Src, Dst>where
T: Copy,
pub fn to_array(&self) -> [T; 16]
Available on crate feature canvas only.
pub fn to_array(&self) -> [T; 16]
canvas only.Returns an array containing this transform’s terms.
The terms are laid out in the same order as they are
specified in Transform3D::new, that is following the
column-major-column-vector matrix notation.
For example the translation terms are found on the 13th, 14th and 15th slots of the array.
pub fn to_array_transposed(&self) -> [T; 16]
Available on crate feature canvas only.
pub fn to_array_transposed(&self) -> [T; 16]
canvas only.Returns an array containing this transform’s terms transposed.
The terms are laid out in transposed order from the same order of
Transform3D::new and Transform3D::to_array, that is following
the row-major-column-vector matrix notation.
For example the translation terms are found at indices 3, 7 and 11 of the array.
pub fn to_arrays(&self) -> [[T; 4]; 4]
Available on crate feature canvas only.
pub fn to_arrays(&self) -> [[T; 4]; 4]
canvas only.Equivalent to to_array with elements packed four at a time
in an array of arrays.
pub fn to_arrays_transposed(&self) -> [[T; 4]; 4]
Available on crate feature canvas only.
pub fn to_arrays_transposed(&self) -> [[T; 4]; 4]
canvas only.Equivalent to to_array_transposed with elements packed
four at a time in an array of arrays.
pub fn from_array(array: [T; 16]) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn from_array(array: [T; 16]) -> Transform3D<T, Src, Dst>
canvas only.Create a transform providing its components via an array of 16 elements instead of as individual parameters.
The order of the components corresponds to the
column-major-column-vector matrix notation (the same order
as Transform3D::new).
pub fn from_arrays(array: [[T; 4]; 4]) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn from_arrays(array: [[T; 4]; 4]) -> Transform3D<T, Src, Dst>
canvas only.Equivalent to from_array with elements packed four at a time
in an array of arrays.
The order of the components corresponds to the
column-major-column-vector matrix notation (the same order
as Transform3D::new).
pub fn from_untyped(
m: &Transform3D<T, UnknownUnit, UnknownUnit>,
) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn from_untyped( m: &Transform3D<T, UnknownUnit, UnknownUnit>, ) -> Transform3D<T, Src, Dst>
canvas only.Tag a unitless value with units.
pub fn to_untyped(&self) -> Transform3D<T, UnknownUnit, UnknownUnit>
Available on crate feature canvas only.
pub fn to_untyped(&self) -> Transform3D<T, UnknownUnit, UnknownUnit>
canvas only.Drop the units, preserving only the numeric value.
pub fn with_source<NewSrc>(&self) -> Transform3D<T, NewSrc, Dst>
Available on crate feature canvas only.
pub fn with_source<NewSrc>(&self) -> Transform3D<T, NewSrc, Dst>
canvas only.Returns the same transform with a different source unit.
pub fn with_destination<NewDst>(&self) -> Transform3D<T, Src, NewDst>
Available on crate feature canvas only.
pub fn with_destination<NewDst>(&self) -> Transform3D<T, Src, NewDst>
canvas only.Returns the same transform with a different destination unit.
pub fn to_2d(&self) -> Transform2D<T, Src, Dst>
Available on crate feature canvas only.
pub fn to_2d(&self) -> Transform2D<T, Src, Dst>
canvas only.Create a 2D transform picking the relevant terms from this transform.
This method assumes that self represents a 2d transformation, callers
should check that is_2d returns true beforehand.
pub fn is_scale_offset_2d_eps(&self, epsilon: T) -> boolwhere
T: Signed + PartialOrd,
Available on crate feature canvas only.
pub fn is_scale_offset_2d_eps(&self, epsilon: T) -> boolwhere
T: Signed + PartialOrd,
canvas only.Returns true if self can be represented as a 2d scale+offset transform.
pub fn to_scale_offset2d(&self) -> Option<ScaleOffset2D<T, Src, Dst>>
Available on crate feature canvas only.
pub fn to_scale_offset2d(&self) -> Option<ScaleOffset2D<T, Src, Dst>>
canvas only.Creates a 2D scale+offset transform from the current transform.
This method assumes that self can be represented as a 2d scale+offset
transformation, callers should check that [is_scale_offset_2d] or
[is_scale_offset_2d_eps] returns true beforehand.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
impl<T, Src, Dst> Transform3D<T, Src, Dst>
pub fn identity() -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn identity() -> Transform3D<T, Src, Dst>
canvas only.Creates an identity matrix:
1 0 0 0
0 1 0 0
0 0 1 0
0 0 0 1pub fn perspective(d: T) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn perspective(d: T) -> Transform3D<T, Src, Dst>
canvas only.Create a simple perspective transform, projecting to the plane z = -d.
1 0 0 0
0 1 0 0
0 0 1 -1/d
0 0 0 1See https://drafts.csswg.org/css-transforms-2/#PerspectiveDefined.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for combining generic transformations
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for combining generic transformations
pub fn then<NewDst>(
&self,
other: &Transform3D<T, Dst, NewDst>,
) -> Transform3D<T, Src, NewDst>
Available on crate feature canvas only.
pub fn then<NewDst>( &self, other: &Transform3D<T, Dst, NewDst>, ) -> Transform3D<T, Src, NewDst>
canvas only.Returns the multiplication of the two matrices such that mat’s transformation applies after self’s transformation.
Assuming row vectors, this is equivalent to self * mat
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for creating and combining translation transformations
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for creating and combining translation transformations
pub fn translation(x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn translation(x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
canvas only.Create a 3d translation transform:
1 0 0 0
0 1 0 0
0 0 1 0
x y z 1pub fn pre_translate(&self, v: Vector3D<T, Src>) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn pre_translate(&self, v: Vector3D<T, Src>) -> Transform3D<T, Src, Dst>
canvas only.Returns a transform with a translation applied before self’s transformation.
pub fn then_translate(&self, v: Vector3D<T, Dst>) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn then_translate(&self, v: Vector3D<T, Dst>) -> Transform3D<T, Src, Dst>
canvas only.Returns a transform with a translation applied after self’s transformation.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for creating and combining scale transformations
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for creating and combining scale transformations
pub fn scale(x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn scale(x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
canvas only.Create a 3d scale transform:
x 0 0 0
0 y 0 0
0 0 z 0
0 0 0 1pub fn pre_scale(&self, x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn pre_scale(&self, x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
canvas only.Returns a transform with a scale applied before self’s transformation.
pub fn then_scale(&self, x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn then_scale(&self, x: T, y: T, z: T) -> Transform3D<T, Src, Dst>
canvas only.Returns a transform with a scale applied after self’s transformation.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for apply transformations to objects
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for apply transformations to objects
pub fn transform_point2d_homogeneous(
&self,
p: Point2D<T, Src>,
) -> HomogeneousVector<T, Dst>
Available on crate feature canvas only.
pub fn transform_point2d_homogeneous( &self, p: Point2D<T, Src>, ) -> HomogeneousVector<T, Dst>
canvas only.Returns the homogeneous vector corresponding to the transformed 2d point.
The input point must be use the unit Src, and the returned point has the unit Dst.
pub fn transform_point2d(&self, p: Point2D<T, Src>) -> Option<Point2D<T, Dst>>
Available on crate feature canvas only.
pub fn transform_point2d(&self, p: Point2D<T, Src>) -> Option<Point2D<T, Dst>>
canvas only.Returns the given 2d point transformed by this transform, if the transform makes sense,
or None otherwise.
The input point must be use the unit Src, and the returned point has the unit Dst.
pub fn transform_vector2d(&self, v: Vector2D<T, Src>) -> Vector2D<T, Dst>
Available on crate feature canvas only.
pub fn transform_vector2d(&self, v: Vector2D<T, Src>) -> Vector2D<T, Dst>
canvas only.Returns the given 2d vector transformed by this matrix.
The input point must be use the unit Src, and the returned point has the unit Dst.
pub fn transform_point3d_homogeneous(
&self,
p: Point3D<T, Src>,
) -> HomogeneousVector<T, Dst>
Available on crate feature canvas only.
pub fn transform_point3d_homogeneous( &self, p: Point3D<T, Src>, ) -> HomogeneousVector<T, Dst>
canvas only.Returns the homogeneous vector corresponding to the transformed 3d point.
The input point must be use the unit Src, and the returned point has the unit Dst.
pub fn transform_point3d(&self, p: Point3D<T, Src>) -> Option<Point3D<T, Dst>>
Available on crate feature canvas only.
pub fn transform_point3d(&self, p: Point3D<T, Src>) -> Option<Point3D<T, Dst>>
canvas only.Returns the given 3d point transformed by this transform, if the transform makes sense,
or None otherwise.
The input point must be use the unit Src, and the returned point has the unit Dst.
pub fn transform_vector3d(&self, v: Vector3D<T, Src>) -> Vector3D<T, Dst>
Available on crate feature canvas only.
pub fn transform_vector3d(&self, v: Vector3D<T, Src>) -> Vector3D<T, Dst>
canvas only.Returns the given 3d vector transformed by this matrix.
The input point must be use the unit Src, and the returned point has the unit Dst.
pub fn outer_transformed_rect(
&self,
rect: &Rect<T, Src>,
) -> Option<Rect<T, Dst>>
Available on crate feature canvas only.
pub fn outer_transformed_rect( &self, rect: &Rect<T, Src>, ) -> Option<Rect<T, Dst>>
canvas only.Returns a rectangle that encompasses the result of transforming the given rectangle by this
transform, if the transform makes sense for it, or None otherwise.
pub fn outer_transformed_box2d(
&self,
b: &Box2D<T, Src>,
) -> Option<Box2D<T, Dst>>
Available on crate feature canvas only.
pub fn outer_transformed_box2d( &self, b: &Box2D<T, Src>, ) -> Option<Box2D<T, Dst>>
canvas only.Returns a 2d box that encompasses the result of transforming the given box by this
transform, if the transform makes sense for it, or None otherwise.
pub fn outer_transformed_box3d(
&self,
b: &Box3D<T, Src>,
) -> Option<Box3D<T, Dst>>
Available on crate feature canvas only.
pub fn outer_transformed_box3d( &self, b: &Box3D<T, Src>, ) -> Option<Box3D<T, Dst>>
canvas only.Returns a 3d box that encompasses the result of transforming the given box by this
transform, if the transform makes sense for it, or None otherwise.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
impl<T, Src, Dst> Transform3D<T, Src, Dst>
pub fn ortho(
left: T,
right: T,
bottom: T,
top: T,
near: T,
far: T,
) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn ortho( left: T, right: T, bottom: T, top: T, near: T, far: T, ) -> Transform3D<T, Src, Dst>
canvas only.Create an orthogonal projection transform.
pub fn is_backface_visible(&self) -> bool
Available on crate feature canvas only.
pub fn is_backface_visible(&self) -> bool
canvas only.Check whether shapes on the XY plane with Z pointing towards the screen transformed by this matrix would be facing back.
pub fn is_invertible(&self) -> bool
Available on crate feature canvas only.
pub fn is_invertible(&self) -> bool
canvas only.Returns whether it is possible to compute the inverse transform.
pub fn inverse(&self) -> Option<Transform3D<T, Dst, Src>>
Available on crate feature canvas only.
pub fn inverse(&self) -> Option<Transform3D<T, Dst, Src>>
canvas only.Returns the inverse transform if possible.
pub fn determinant(&self) -> T
Available on crate feature canvas only.
pub fn determinant(&self) -> T
canvas only.Compute the determinant of the transform.
pub fn mul_s(&self, x: T) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn mul_s(&self, x: T) -> Transform3D<T, Src, Dst>
canvas only.Multiplies all of the transform’s component by a scalar and returns the result.
pub fn from_scale(scale: Scale<T, Src, Dst>) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn from_scale(scale: Scale<T, Src, Dst>) -> Transform3D<T, Src, Dst>
canvas only.Convenience function to create a scale transform from a Scale.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
impl<T, Src, Dst> Transform3D<T, Src, Dst>
pub fn project_to_2d(&self) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn project_to_2d(&self) -> Transform3D<T, Src, Dst>
canvas only.Returns a projection of this transform in 2d space.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
impl<T, Src, Dst> Transform3D<T, Src, Dst>
pub fn cast<NewT>(&self) -> Transform3D<NewT, Src, Dst>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn cast<NewT>(&self) -> Transform3D<NewT, Src, Dst>where
NewT: NumCast,
canvas only.Cast from one numeric representation to another, preserving the units.
pub fn try_cast<NewT>(&self) -> Option<Transform3D<NewT, Src, Dst>>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn try_cast<NewT>(&self) -> Option<Transform3D<NewT, Src, Dst>>where
NewT: NumCast,
canvas only.Fallible cast from one numeric representation to another, preserving the units.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
impl<T, Src, Dst> Transform3D<T, Src, Dst>
pub fn is_scale_offset_2d(&self) -> bool
Available on crate feature canvas only.
pub fn is_scale_offset_2d(&self) -> bool
canvas only.Returns true if self can be represented as a 2d scale+offset
transform, using T’s default epsilon value.
pub fn skew(alpha: Angle<T>, beta: Angle<T>) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn skew(alpha: Angle<T>, beta: Angle<T>) -> Transform3D<T, Src, Dst>
canvas only.Create a 2d skew transform.
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for creating and combining rotation transformations
impl<T, Src, Dst> Transform3D<T, Src, Dst>
Methods for creating and combining rotation transformations
pub fn rotation(x: T, y: T, z: T, theta: Angle<T>) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn rotation(x: T, y: T, z: T, theta: Angle<T>) -> Transform3D<T, Src, Dst>
canvas only.Create a 3d rotation transform from an angle / axis. The supplied axis must be normalized.
pub fn then_rotate(
&self,
x: T,
y: T,
z: T,
theta: Angle<T>,
) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn then_rotate( &self, x: T, y: T, z: T, theta: Angle<T>, ) -> Transform3D<T, Src, Dst>
canvas only.Returns a transform with a rotation applied after self’s transformation.
pub fn pre_rotate(
&self,
x: T,
y: T,
z: T,
theta: Angle<T>,
) -> Transform3D<T, Src, Dst>
Available on crate feature canvas only.
pub fn pre_rotate( &self, x: T, y: T, z: T, theta: Angle<T>, ) -> Transform3D<T, Src, Dst>
canvas only.Returns a transform with a rotation applied before self’s transformation.
impl<T, Src, Dst> Transform3D<T, Src, Dst>where
T: ApproxEq<T>,
impl<T, Src, Dst> Transform3D<T, Src, Dst>where
T: ApproxEq<T>,
pub fn approx_eq(&self, other: &Transform3D<T, Src, Dst>) -> bool
Available on crate feature canvas only.
pub fn approx_eq(&self, other: &Transform3D<T, Src, Dst>) -> bool
canvas only.Returns true if this transform is approximately equal to the other one, using
T’s default epsilon value.
The same as ApproxEq::approx_eq but available without importing trait.
pub fn approx_eq_eps(&self, other: &Transform3D<T, Src, Dst>, eps: &T) -> bool
Available on crate feature canvas only.
pub fn approx_eq_eps(&self, other: &Transform3D<T, Src, Dst>, eps: &T) -> bool
canvas only.Returns true if this transform is approximately equal to the other one, using
a provided epsilon value.
The same as ApproxEq::approx_eq_eps but available without importing trait.