Struct Translation2D
pub struct Translation2D<T, Src, Dst> {
pub x: T,
pub y: T,
/* private fields */
}geometry only.Expand description
A 2d transformation from a space to another that can only express translations.
The main benefit of this type over a Vector2D is the ability to cast
between source and destination spaces.
Example:
use euclid::{Translation2D, Point2D, point2};
struct ParentSpace;
struct ChildSpace;
type ScrollOffset = Translation2D<i32, ParentSpace, ChildSpace>;
type ParentPoint = Point2D<i32, ParentSpace>;
type ChildPoint = Point2D<i32, ChildSpace>;
let scrolling = ScrollOffset::new(0, 100);
let p1: ParentPoint = point2(0, 0);
let p2: ChildPoint = scrolling.transform_point(p1);Fields
x: Ty: TImplementations
impl<T, Src, Dst> Translation2D<T, Src, Dst>
impl<T, Src, Dst> Translation2D<T, Src, Dst>
pub const fn new(x: T, y: T) -> Translation2D<T, Src, Dst>
canvas only.pub fn splat(v: T) -> Translation2D<T, Src, Dst>where
T: Clone,
canvas only.pub fn identity() -> Translation2D<T, Src, Dst>where
T: Zero,
Available on crate feature canvas only.
pub fn identity() -> Translation2D<T, Src, Dst>where
T: Zero,
canvas only.Creates no-op translation (both x and y is zero()).
pub fn is_identity(&self) -> bool
Available on crate feature canvas only.
pub fn is_identity(&self) -> bool
canvas only.Check if translation does nothing (both x and y is zero()).
use euclid::default::Translation2D;
assert_eq!(Translation2D::<f32>::identity().is_identity(), true);
assert_eq!(Translation2D::new(0, 0).is_identity(), true);
assert_eq!(Translation2D::new(1, 0).is_identity(), false);
assert_eq!(Translation2D::new(0, 1).is_identity(), false);pub fn transform_size(&self, s: Size2D<T, Src>) -> Size2D<T, Dst>
Available on crate feature canvas only.
pub fn transform_size(&self, s: Size2D<T, Src>) -> Size2D<T, Dst>
canvas only.No-op, just cast the unit.
impl<T, Src, Dst> Translation2D<T, Src, Dst>where
T: Copy,
impl<T, Src, Dst> Translation2D<T, Src, Dst>where
T: Copy,
pub fn to_vector(&self) -> Vector2D<T, Src>
Available on crate feature canvas only.
pub fn to_vector(&self) -> Vector2D<T, Src>
canvas only.Cast into a 2D vector.
pub fn to_array(&self) -> [T; 2]
Available on crate feature canvas only.
pub fn to_array(&self) -> [T; 2]
canvas only.Cast into an array with x and y.
pub fn to_tuple(&self) -> (T, T)
Available on crate feature canvas only.
pub fn to_tuple(&self) -> (T, T)
canvas only.Cast into a tuple with x and y.
pub fn to_untyped(&self) -> Translation2D<T, UnknownUnit, UnknownUnit>
Available on crate feature canvas only.
pub fn to_untyped(&self) -> Translation2D<T, UnknownUnit, UnknownUnit>
canvas only.Drop the units, preserving only the numeric value.
pub fn from_untyped(
t: &Translation2D<T, UnknownUnit, UnknownUnit>,
) -> Translation2D<T, Src, Dst>
Available on crate feature canvas only.
pub fn from_untyped( t: &Translation2D<T, UnknownUnit, UnknownUnit>, ) -> Translation2D<T, Src, Dst>
canvas only.Tag a unitless value with units.
pub fn to_transform(&self) -> Transform2D<T, Src, Dst>
Available on crate feature canvas only.
pub fn to_transform(&self) -> Transform2D<T, Src, Dst>
canvas only.Returns the matrix representation of this translation.
pub fn transform_point(
&self,
p: Point2D<T, Src>,
) -> Point2D<<T as Add>::Output, Dst>where
T: Add,
Available on crate feature canvas only.
pub fn transform_point(
&self,
p: Point2D<T, Src>,
) -> Point2D<<T as Add>::Output, Dst>where
T: Add,
canvas only.Translate a point and cast its unit.
pub fn transform_rect(&self, r: &Rect<T, Src>) -> Rect<<T as Add>::Output, Dst>where
T: Add<Output = T>,
Available on crate feature canvas only.
pub fn transform_rect(&self, r: &Rect<T, Src>) -> Rect<<T as Add>::Output, Dst>where
T: Add<Output = T>,
canvas only.Translate a rectangle and cast its unit.
pub fn transform_box(&self, r: &Box2D<T, Src>) -> Box2D<<T as Add>::Output, Dst>where
T: Add,
Available on crate feature canvas only.
pub fn transform_box(&self, r: &Box2D<T, Src>) -> Box2D<<T as Add>::Output, Dst>where
T: Add,
canvas only.Translate a 2D box and cast its unit.
pub fn inverse(&self) -> Translation2D<<T as Neg>::Output, Dst, Src>where
T: Neg,
Available on crate feature canvas only.
pub fn inverse(&self) -> Translation2D<<T as Neg>::Output, Dst, Src>where
T: Neg,
canvas only.Return the inverse transformation.
impl<T, Src, Dst> Translation2D<T, Src, Dst>
impl<T, Src, Dst> Translation2D<T, Src, Dst>
pub fn cast<NewT>(self) -> Translation2D<NewT, Src, Dst>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn cast<NewT>(self) -> Translation2D<NewT, Src, Dst>where
NewT: NumCast,
canvas only.Cast from one numeric representation to another, preserving the units.
When casting from floating vector to integer coordinates, the decimals are truncated
as one would expect from a simple cast, but this behavior does not always make sense
geometrically. Consider using round(), ceil() or floor() before casting.
pub fn try_cast<NewT>(self) -> Option<Translation2D<NewT, Src, Dst>>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn try_cast<NewT>(self) -> Option<Translation2D<NewT, Src, Dst>>where
NewT: NumCast,
canvas only.Fallible cast from one numeric representation to another, preserving the units.
When casting from floating vector to integer coordinates, the decimals are truncated
as one would expect from a simple cast, but this behavior does not always make sense
geometrically. Consider using round(), ceil() or floor() before casting.
pub fn to_f32(self) -> Translation2D<f32, Src, Dst>
Available on crate feature canvas only.
pub fn to_f32(self) -> Translation2D<f32, Src, Dst>
canvas only.Cast into an f32 vector.
pub fn to_f64(self) -> Translation2D<f64, Src, Dst>
Available on crate feature canvas only.
pub fn to_f64(self) -> Translation2D<f64, Src, Dst>
canvas only.Cast into an f64 vector.
pub fn to_usize(self) -> Translation2D<usize, Src, Dst>
Available on crate feature canvas only.
pub fn to_usize(self) -> Translation2D<usize, Src, Dst>
canvas only.Cast into an usize vector, truncating decimals if any.
When casting from floating vector vectors, it is worth considering whether
to round(), ceil() or floor() before the cast in order to obtain
the desired conversion behavior.
pub fn to_u32(self) -> Translation2D<u32, Src, Dst>
Available on crate feature canvas only.
pub fn to_u32(self) -> Translation2D<u32, Src, Dst>
canvas only.Cast into an u32 vector, truncating decimals if any.
When casting from floating vector vectors, it is worth considering whether
to round(), ceil() or floor() before the cast in order to obtain
the desired conversion behavior.
pub fn to_i32(self) -> Translation2D<i32, Src, Dst>
Available on crate feature canvas only.
pub fn to_i32(self) -> Translation2D<i32, Src, Dst>
canvas only.Cast into an i32 vector, truncating decimals if any.
When casting from floating vector vectors, it is worth considering whether
to round(), ceil() or floor() before the cast in order to obtain
the desired conversion behavior.
pub fn to_i64(self) -> Translation2D<i64, Src, Dst>
Available on crate feature canvas only.
pub fn to_i64(self) -> Translation2D<i64, Src, Dst>
canvas only.Cast into an i64 vector, truncating decimals if any.
When casting from floating vector vectors, it is worth considering whether
to round(), ceil() or floor() before the cast in order to obtain
the desired conversion behavior.