Vector3D

Struct Vector3D 

pub struct Vector3D<T, U> {
    pub x: T,
    pub y: T,
    pub z: T,
    /* private fields */
}
Available on crate feature geometry only.
A 3d Vector tagged with a unit.

Fields

x: T

The x (traditionally, horizontal) coordinate.

y: T

The y (traditionally, vertical) coordinate.

z: T

The z (traditionally, depth) coordinate.

Implementations



impl<T, U> Vector3D<T, U>

pub fn zero() -> Vector3D<T, U>
where T: Zero,

Available on crate feature canvas only.

Constructor, setting all components to zero.

pub fn one() -> Vector3D<T, U>
where T: One,

Available on crate feature canvas only.

Constructor, setting all components to one.

pub const fn new(x: T, y: T, z: T) -> Vector3D<T, U>

Available on crate feature canvas only.

Constructor taking scalar values directly.

pub fn splat(v: T) -> Vector3D<T, U>
where T: Clone,

Available on crate feature canvas only.

Constructor setting all components to the same value.

pub fn from_lengths( x: Length<T, U>, y: Length<T, U>, z: Length<T, U>, ) -> Vector3D<T, U>

Available on crate feature canvas only.

Constructor taking properly Lengths instead of scalar values.

pub fn from_untyped(p: Vector3D<T, UnknownUnit>) -> Vector3D<T, U>

Available on crate feature canvas only.

Tag a unitless value with units.

pub fn map<V, F>(self, f: F) -> Vector3D<V, U>
where F: FnMut(T) -> V,

Available on crate feature canvas only.

Apply the function f to each component of this vector.

Example

This may be used to perform unusual arithmetic which is not already offered as methods.

use euclid::default::Vector3D;

let p = Vector3D::<u32>::new(5, 11, 15);
assert_eq!(p.map(|coord| coord.saturating_sub(10)), Vector3D::new(0, 1, 5));

pub fn zip<V, F>(self, rhs: Vector3D<T, U>, f: F) -> Vector3D<V, U>
where F: FnMut(T, T) -> V,

Available on crate feature canvas only.

Apply the function f to each pair of components of this point and rhs.

Example

This may be used to perform unusual arithmetic which is not already offered as methods.

use euclid::default::Vector3D;

let a: Vector3D<u8> = Vector3D::new(50, 200, 10);
let b: Vector3D<u8> = Vector3D::new(100, 100, 0);
assert_eq!(a.zip(b, u8::saturating_add), Vector3D::new(150, 255, 10));

pub fn abs(self) -> Vector3D<T, U>
where T: Signed,

Available on crate feature canvas only.

Computes the vector with absolute values of each component.

Example
enum U {}

assert_eq!(vec3::<_, U>(-1, 0, 2).abs(), vec3(1, 0, 2));

let vec = vec3::<_, U>(f32::NAN, 0.0, -f32::MAX).abs();
assert!(vec.x.is_nan());
assert_eq!(vec.y, 0.0);
assert_eq!(vec.z, f32::MAX);
Panics

The behavior for each component follows the scalar type’s implementation of num_traits::Signed::abs.

pub fn dot(self, other: Vector3D<T, U>) -> T
where T: Add<Output = T> + Mul<Output = T>,

Available on crate feature canvas only.

Dot product.



impl<T, U> Vector3D<T, U>
where T: Copy,

pub fn cross(self, other: Vector3D<T, U>) -> Vector3D<T, U>
where T: Sub<Output = T> + Mul<Output = T>,

Available on crate feature canvas only.

Cross product.

pub fn component_mul(self, other: Vector3D<T, U>) -> Vector3D<T, U>
where T: Mul<Output = T>,

Available on crate feature canvas only.

Returns the component-wise multiplication of the two vectors.

pub fn component_div(self, other: Vector3D<T, U>) -> Vector3D<T, U>
where T: Div<Output = T>,

Available on crate feature canvas only.

Returns the component-wise division of the two vectors.

pub fn to_point(self) -> Point3D<T, U>

Available on crate feature canvas only.

Cast this vector into a point.

Equivalent to adding this vector to the origin.

pub fn xy(self) -> Vector2D<T, U>

Available on crate feature canvas only.

Returns a 2d vector using this vector’s x and y coordinates

pub fn xz(self) -> Vector2D<T, U>

Available on crate feature canvas only.

Returns a 2d vector using this vector’s x and z coordinates

pub fn yz(self) -> Vector2D<T, U>

Available on crate feature canvas only.

Returns a 2d vector using this vector’s x and z coordinates

pub fn to_array(self) -> [T; 3]

Available on crate feature canvas only.

Cast into an array with x, y and z.

pub fn to_array_4d(self) -> [T; 4]
where T: Zero,

Available on crate feature canvas only.

Cast into an array with x, y, z and 0.

pub fn to_tuple(self) -> (T, T, T)

Available on crate feature canvas only.

Cast into a tuple with x, y and z.

pub fn to_tuple_4d(self) -> (T, T, T, T)
where T: Zero,

Available on crate feature canvas only.

Cast into a tuple with x, y, z and 0.

pub fn to_untyped(self) -> Vector3D<T, UnknownUnit>

Available on crate feature canvas only.

Drop the units, preserving only the numeric value.

pub fn cast_unit<V>(self) -> Vector3D<T, V>

Available on crate feature canvas only.

Cast the unit.

pub fn to_2d(self) -> Vector2D<T, U>

Available on crate feature canvas only.

Convert into a 2d vector.

pub fn round(self) -> Vector3D<T, U>
where T: Round,

Available on crate feature canvas only.

Rounds each component to the nearest integer value.

This behavior is preserved for negative values (unlike the basic cast).

enum Mm {}

assert_eq!(vec3::<_, Mm>(-0.1, -0.8, 0.4).round(), vec3::<_, Mm>(0.0, -1.0, 0.0))

pub fn ceil(self) -> Vector3D<T, U>
where T: Ceil,

Available on crate feature canvas only.

Rounds each component to the smallest integer equal or greater than the original value.

This behavior is preserved for negative values (unlike the basic cast).

enum Mm {}

assert_eq!(vec3::<_, Mm>(-0.1, -0.8, 0.4).ceil(), vec3::<_, Mm>(0.0, 0.0, 1.0))

pub fn floor(self) -> Vector3D<T, U>
where T: Floor,

Available on crate feature canvas only.

Rounds each component to the biggest integer equal or lower than the original value.

This behavior is preserved for negative values (unlike the basic cast).

enum Mm {}

assert_eq!(vec3::<_, Mm>(-0.1, -0.8, 0.4).floor(), vec3::<_, Mm>(-1.0, -1.0, 0.0))

pub fn to_transform(self) -> Transform3D<T, U, U>
where T: Zero + One,

Available on crate feature canvas only.

Creates translation by this vector in vector units



impl<T, U> Vector3D<T, U>
where T: Copy + Mul<Output = T> + Add<Output = T>,

pub fn square_length(self) -> T

Available on crate feature canvas only.

Returns the vector’s length squared.

pub fn project_onto_vector(self, onto: Vector3D<T, U>) -> Vector3D<T, U>
where T: Sub<Output = T> + Div<Output = T>,

Available on crate feature canvas only.

Returns this vector projected onto another one.

Projecting onto a nil vector will cause a division by zero.



impl<T, U> Vector3D<T, U>
where T: Float,

pub fn robust_normalize(self) -> Vector3D<T, U>

Available on crate feature canvas only.

Return the normalized vector even if the length is larger than the max value of Float.

pub fn is_finite(self) -> bool

Available on crate feature canvas only.

Returns true if all members are finite.



impl<T, U> Vector3D<T, U>
where T: Real,

pub fn angle_to(self, other: Vector3D<T, U>) -> Angle<T>
where T: Trig,

Available on crate feature canvas only.

Returns the positive angle between this vector and another vector.

The returned angle is between 0 and PI.

pub fn length(self) -> T

Available on crate feature canvas only.

Returns the vector length.

pub fn normalize(self) -> Vector3D<T, U>

Available on crate feature canvas only.

Returns the vector with length of one unit

pub fn try_normalize(self) -> Option<Vector3D<T, U>>

Available on crate feature canvas only.

Returns the vector with length of one unit.

Unlike [Vector2D::normalize], this returns None in the case that the length of the vector is zero.

pub fn with_max_length(self, max_length: T) -> Vector3D<T, U>

Available on crate feature canvas only.

Return this vector capped to a maximum length.

pub fn with_min_length(self, min_length: T) -> Vector3D<T, U>

Available on crate feature canvas only.

Return this vector with a minimum length applied.

pub fn clamp_length(self, min: T, max: T) -> Vector3D<T, U>

Available on crate feature canvas only.

Return this vector with minimum and maximum lengths applied.



impl<T, U> Vector3D<T, U>
where T: One + Add<Output = T> + Sub<Output = T> + Mul<Output = T> + Copy,

pub fn lerp(self, other: Vector3D<T, U>, t: T) -> Vector3D<T, U>

Available on crate feature canvas only.

Linearly interpolate each component between this vector and another vector.

Example
use euclid::vec3;
use euclid::default::Vector3D;

let from: Vector3D<_> = vec3(0.0, 10.0, -1.0);
let to:  Vector3D<_> = vec3(8.0, -4.0,  0.0);

assert_eq!(from.lerp(to, -1.0), vec3(-8.0,  24.0, -2.0));
assert_eq!(from.lerp(to,  0.0), vec3( 0.0,  10.0, -1.0));
assert_eq!(from.lerp(to,  0.5), vec3( 4.0,   3.0, -0.5));
assert_eq!(from.lerp(to,  1.0), vec3( 8.0,  -4.0,  0.0));
assert_eq!(from.lerp(to,  2.0), vec3(16.0, -18.0,  1.0));

pub fn reflect(self, normal: Vector3D<T, U>) -> Vector3D<T, U>

Available on crate feature canvas only.

Returns a reflection vector using an incident ray and a surface normal.



impl<T, U> Vector3D<T, U>
where T: PartialOrd,

pub fn min(self, other: Vector3D<T, U>) -> Vector3D<T, U>

Available on crate feature canvas only.

Returns the vector each component of which are minimum of this vector and another.

pub fn max(self, other: Vector3D<T, U>) -> Vector3D<T, U>

Available on crate feature canvas only.

Returns the vector each component of which are maximum of this vector and another.

pub fn clamp(self, start: Vector3D<T, U>, end: Vector3D<T, U>) -> Vector3D<T, U>
where T: Copy,

Available on crate feature canvas only.

Returns the vector each component of which is clamped by corresponding components of start and end.

Shortcut for self.max(start).min(end).

pub fn greater_than(self, other: Vector3D<T, U>) -> BoolVector3D

Available on crate feature canvas only.

Returns vector with results of “greater than” operation on each component.

pub fn lower_than(self, other: Vector3D<T, U>) -> BoolVector3D

Available on crate feature canvas only.

Returns vector with results of “lower than” operation on each component.



impl<T, U> Vector3D<T, U>
where T: PartialEq,

pub fn equal(self, other: Vector3D<T, U>) -> BoolVector3D

Available on crate feature canvas only.

Returns vector with results of “equal” operation on each component.

pub fn not_equal(self, other: Vector3D<T, U>) -> BoolVector3D

Available on crate feature canvas only.

Returns vector with results of “not equal” operation on each component.



impl<T, U> Vector3D<T, U>
where T: NumCast + Copy,

pub fn cast<NewT>(self) -> Vector3D<NewT, U>
where NewT: NumCast,

Available on crate feature canvas only.

Cast from one numeric representation to another, preserving the units.

When casting from floating vector to integer coordinates, the decimals are truncated as one would expect from a simple cast, but this behavior does not always make sense geometrically. Consider using round(), ceil() or floor() before casting.

pub fn try_cast<NewT>(self) -> Option<Vector3D<NewT, U>>
where NewT: NumCast,

Available on crate feature canvas only.

Fallible cast from one numeric representation to another, preserving the units.

When casting from floating vector to integer coordinates, the decimals are truncated as one would expect from a simple cast, but this behavior does not always make sense geometrically. Consider using round(), ceil() or floor() before casting.

pub fn to_f32(self) -> Vector3D<f32, U>

Available on crate feature canvas only.

Cast into an f32 vector.

pub fn to_f64(self) -> Vector3D<f64, U>

Available on crate feature canvas only.

Cast into an f64 vector.

pub fn to_usize(self) -> Vector3D<usize, U>

Available on crate feature canvas only.

Cast into an usize vector, truncating decimals if any.

When casting from floating vector vectors, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.

pub fn to_isize(self) -> Vector3D<isize, U>

Available on crate feature canvas only.

Cast into an isize vector, truncating decimals if any.

When casting from floating vector vectors, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.

pub fn to_u32(self) -> Vector3D<u32, U>

Available on crate feature canvas only.

Cast into an u32 vector, truncating decimals if any.

When casting from floating vector vectors, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.

pub fn to_i32(self) -> Vector3D<i32, U>

Available on crate feature canvas only.

Cast into an i32 vector, truncating decimals if any.

When casting from floating vector vectors, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.

pub fn to_i64(self) -> Vector3D<i64, U>

Available on crate feature canvas only.

Cast into an i64 vector, truncating decimals if any.

When casting from floating vector vectors, it is worth considering whether to round(), ceil() or floor() before the cast in order to obtain the desired conversion behavior.