Struct Scale
pub struct Scale<T, Src, Dst>(pub T, _);geometry only.Expand description
A scaling factor between two different units of measurement.
This is effectively a type-safe float, intended to be used in combination with other types like
length::Length to enforce conversion between systems of measurement at compile time.
Src and Dst represent the units before and after multiplying a value by a Scale. They
may be types without values, such as empty enums. For example:
use euclid::Scale;
use euclid::Length;
enum Mm {};
enum Inch {};
let mm_per_inch: Scale<f32, Inch, Mm> = Scale::new(25.4);
let one_foot: Length<f32, Inch> = Length::new(12.0);
let one_foot_in_mm: Length<f32, Mm> = one_foot * mm_per_inch;Tuple Fields
0: TImplementations
impl<T, Src, Dst> Scale<T, Src, Dst>
impl<T, Src, Dst> Scale<T, Src, Dst>
pub const fn new(x: T) -> Scale<T, Src, Dst>
canvas only.pub fn identity() -> Scale<T, Src, Dst>where
T: One,
Available on crate feature canvas only.
pub fn identity() -> Scale<T, Src, Dst>where
T: One,
canvas only.Creates an identity scale (1.0).
pub fn transform_point(
self,
point: Point2D<T, Src>,
) -> Point2D<<T as Mul>::Output, Dst>
Available on crate feature canvas only.
pub fn transform_point( self, point: Point2D<T, Src>, ) -> Point2D<<T as Mul>::Output, Dst>
canvas only.Returns the given point transformed by this scale.
Example
use euclid::{Scale, point2};
enum Mm {};
enum Cm {};
let to_mm: Scale<i32, Cm, Mm> = Scale::new(10);
assert_eq!(to_mm.transform_point(point2(42, -42)), point2(420, -420));pub fn transform_point3d(
self,
point: Point3D<T, Src>,
) -> Point3D<<T as Mul>::Output, Dst>
Available on crate feature canvas only.
pub fn transform_point3d( self, point: Point3D<T, Src>, ) -> Point3D<<T as Mul>::Output, Dst>
canvas only.Returns the given point transformed by this scale.
pub fn transform_vector(
self,
vec: Vector2D<T, Src>,
) -> Vector2D<<T as Mul>::Output, Dst>
Available on crate feature canvas only.
pub fn transform_vector( self, vec: Vector2D<T, Src>, ) -> Vector2D<<T as Mul>::Output, Dst>
canvas only.Returns the given vector transformed by this scale.
Example
use euclid::{Scale, vec2};
enum Mm {};
enum Cm {};
let to_mm: Scale<i32, Cm, Mm> = Scale::new(10);
assert_eq!(to_mm.transform_vector(vec2(42, -42)), vec2(420, -420));pub fn transform_size(
self,
size: Size2D<T, Src>,
) -> Size2D<<T as Mul>::Output, Dst>
Available on crate feature canvas only.
pub fn transform_size( self, size: Size2D<T, Src>, ) -> Size2D<<T as Mul>::Output, Dst>
canvas only.Returns the given size transformed by this scale.
Example
use euclid::{Scale, size2};
enum Mm {};
enum Cm {};
let to_mm: Scale<i32, Cm, Mm> = Scale::new(10);
assert_eq!(to_mm.transform_size(size2(42, -42)), size2(420, -420));pub fn transform_rect(
self,
rect: &Rect<T, Src>,
) -> Rect<<T as Mul>::Output, Dst>
Available on crate feature canvas only.
pub fn transform_rect( self, rect: &Rect<T, Src>, ) -> Rect<<T as Mul>::Output, Dst>
canvas only.Returns the given rect transformed by this scale.
Example
use euclid::{Scale, rect};
enum Mm {};
enum Cm {};
let to_mm: Scale<i32, Cm, Mm> = Scale::new(10);
assert_eq!(to_mm.transform_rect(&rect(1, 2, 42, -42)), rect(10, 20, 420, -420));pub fn transform_box2d(
self,
b: &Box2D<T, Src>,
) -> Box2D<<T as Mul>::Output, Dst>
Available on crate feature canvas only.
pub fn transform_box2d( self, b: &Box2D<T, Src>, ) -> Box2D<<T as Mul>::Output, Dst>
canvas only.Returns the given box transformed by this scale.
pub fn transform_box3d(
self,
b: &Box3D<T, Src>,
) -> Box3D<<T as Mul>::Output, Dst>
Available on crate feature canvas only.
pub fn transform_box3d( self, b: &Box3D<T, Src>, ) -> Box3D<<T as Mul>::Output, Dst>
canvas only.Returns the given box transformed by this scale.
pub fn is_identity(self) -> bool
Available on crate feature canvas only.
pub fn is_identity(self) -> bool
canvas only.Returns true if this scale has no effect.
Example
use euclid::Scale;
use euclid::num::One;
enum Mm {};
enum Cm {};
let cm_per_mm: Scale<f32, Mm, Cm> = Scale::new(0.1);
let mm_per_mm: Scale<f32, Mm, Mm> = Scale::new(1.0);
assert_eq!(cm_per_mm.is_identity(), false);
assert_eq!(mm_per_mm.is_identity(), true);
assert_eq!(mm_per_mm, Scale::one());pub fn get(self) -> T
Available on crate feature canvas only.
pub fn get(self) -> T
canvas only.Returns the underlying scalar scale factor.
pub fn inverse(self) -> Scale<<T as Div>::Output, Dst, Src>
Available on crate feature canvas only.
pub fn inverse(self) -> Scale<<T as Div>::Output, Dst, Src>
canvas only.The inverse Scale (1.0 / self).
Example
use euclid::Scale;
enum Mm {};
enum Cm {};
let cm_per_mm: Scale<f32, Cm, Mm> = Scale::new(0.1);
assert_eq!(cm_per_mm.inverse(), Scale::new(10.0));pub fn with_source<NewSrc>(self) -> Scale<T, NewSrc, Dst>
Available on crate feature canvas only.
pub fn with_source<NewSrc>(self) -> Scale<T, NewSrc, Dst>
canvas only.Returns the same transform with a different source unit.
pub fn with_destination<NewDst>(self) -> Scale<T, Src, NewDst>
Available on crate feature canvas only.
pub fn with_destination<NewDst>(self) -> Scale<T, Src, NewDst>
canvas only.Returns the same transform with a different destination unit.
impl<T, Src, Dst> Scale<T, Src, Dst>where
T: PartialOrd,
impl<T, Src, Dst> Scale<T, Src, Dst>where
T: PartialOrd,
impl<T, Src, Dst> Scale<T, Src, Dst>where
T: NumCast,
impl<T, Src, Dst> Scale<T, Src, Dst>where
T: NumCast,
pub fn cast<NewT>(self) -> Scale<NewT, Src, Dst>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn cast<NewT>(self) -> Scale<NewT, Src, Dst>where
NewT: NumCast,
canvas only.Cast from one numeric representation to another, preserving the units.
Panics
If the source value cannot be represented by the target type NewT, then
method panics. Use try_cast if that must be case.
Example
use euclid::Scale;
enum Mm {};
enum Cm {};
let to_mm: Scale<i32, Cm, Mm> = Scale::new(10);
assert_eq!(to_mm.cast::<f32>(), Scale::new(10.0));That conversion will panic, because i32 not enough to store such big numbers:
use euclid::Scale;
enum Mm {};// millimeter = 10^-2 meters
enum Em {};// exameter = 10^18 meters
// Panics
let to_em: Scale<i32, Mm, Em> = Scale::new(10e20).cast();pub fn try_cast<NewT>(self) -> Option<Scale<NewT, Src, Dst>>where
NewT: NumCast,
Available on crate feature canvas only.
pub fn try_cast<NewT>(self) -> Option<Scale<NewT, Src, Dst>>where
NewT: NumCast,
canvas only.Fallible cast from one numeric representation to another, preserving the units.
If the source value cannot be represented by the target type NewT, then None
is returned.
Example
use euclid::Scale;
enum Mm {};
enum Cm {};
enum Em {};// Exameter = 10^18 meters
let to_mm: Scale<i32, Cm, Mm> = Scale::new(10);
let to_em: Scale<f32, Mm, Em> = Scale::new(10e20);
assert_eq!(to_mm.try_cast::<f32>(), Some(Scale::new(10.0)));
// Integer to small to store that number
assert_eq!(to_em.try_cast::<i32>(), None);Trait Implementations
impl<T, U1, U2> Div<Scale<T, U1, U2>> for SideOffsets2D<T, U2>
impl<T, U1, U2> Div<Scale<T, U1, U2>> for SideOffsets2D<T, U2>
impl<T, U> DivAssign<Scale<T, U, U>> for Box2D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Box2D<T, U>
fn div_assign(&mut self, scale: Scale<T, U, U>)
fn div_assign(&mut self, scale: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Box3D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Box3D<T, U>
fn div_assign(&mut self, scale: Scale<T, U, U>)
fn div_assign(&mut self, scale: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Point2D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Point2D<T, U>
fn div_assign(&mut self, scale: Scale<T, U, U>)
fn div_assign(&mut self, scale: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Point3D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Point3D<T, U>
fn div_assign(&mut self, scale: Scale<T, U, U>)
fn div_assign(&mut self, scale: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Rect<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Rect<T, U>
fn div_assign(&mut self, scale: Scale<T, U, U>)
fn div_assign(&mut self, scale: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for SideOffsets2D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for SideOffsets2D<T, U>
fn div_assign(&mut self, other: Scale<T, U, U>)
fn div_assign(&mut self, other: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Size2D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Size2D<T, U>
fn div_assign(&mut self, other: Scale<T, U, U>)
fn div_assign(&mut self, other: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Size3D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Size3D<T, U>
fn div_assign(&mut self, other: Scale<T, U, U>)
fn div_assign(&mut self, other: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Vector2D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Vector2D<T, U>
fn div_assign(&mut self, scale: Scale<T, U, U>)
fn div_assign(&mut self, scale: Scale<T, U, U>)
/= operation. Read moreimpl<T, U> DivAssign<Scale<T, U, U>> for Vector3D<T, U>
impl<T, U> DivAssign<Scale<T, U, U>> for Vector3D<T, U>
fn div_assign(&mut self, scale: Scale<T, U, U>)
fn div_assign(&mut self, scale: Scale<T, U, U>)
/= operation. Read moreimpl<T, Src, Dst> From<Scale<T, Src, Dst>> for ScaleOffset2D<T, Src, Dst>
impl<T, Src, Dst> From<Scale<T, Src, Dst>> for ScaleOffset2D<T, Src, Dst>
fn from(s: Scale<T, Src, Dst>) -> ScaleOffset2D<T, Src, Dst>
fn from(s: Scale<T, Src, Dst>) -> ScaleOffset2D<T, Src, Dst>
impl<T, Src, Dst> From<Scale<T, Src, Dst>> for Transform2D<T, Src, Dst>
impl<T, Src, Dst> From<Scale<T, Src, Dst>> for Transform2D<T, Src, Dst>
fn from(s: Scale<T, Src, Dst>) -> Transform2D<T, Src, Dst>
fn from(s: Scale<T, Src, Dst>) -> Transform2D<T, Src, Dst>
impl<T, Src, Dst> From<Scale<T, Src, Dst>> for Transform3D<T, Src, Dst>
impl<T, Src, Dst> From<Scale<T, Src, Dst>> for Transform3D<T, Src, Dst>
fn from(s: Scale<T, Src, Dst>) -> Transform3D<T, Src, Dst>
fn from(s: Scale<T, Src, Dst>) -> Transform3D<T, Src, Dst>
impl<T, U1, U2> Mul<Scale<T, U1, U2>> for SideOffsets2D<T, U1>
impl<T, U1, U2> Mul<Scale<T, U1, U2>> for SideOffsets2D<T, U1>
impl<T, U> MulAssign<Scale<T, U, U>> for Box2D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Box2D<T, U>
fn mul_assign(&mut self, scale: Scale<T, U, U>)
fn mul_assign(&mut self, scale: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Box3D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Box3D<T, U>
fn mul_assign(&mut self, scale: Scale<T, U, U>)
fn mul_assign(&mut self, scale: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Point2D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Point2D<T, U>
fn mul_assign(&mut self, scale: Scale<T, U, U>)
fn mul_assign(&mut self, scale: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Point3D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Point3D<T, U>
fn mul_assign(&mut self, scale: Scale<T, U, U>)
fn mul_assign(&mut self, scale: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Rect<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Rect<T, U>
fn mul_assign(&mut self, scale: Scale<T, U, U>)
fn mul_assign(&mut self, scale: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for SideOffsets2D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for SideOffsets2D<T, U>
fn mul_assign(&mut self, other: Scale<T, U, U>)
fn mul_assign(&mut self, other: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Size2D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Size2D<T, U>
fn mul_assign(&mut self, other: Scale<T, U, U>)
fn mul_assign(&mut self, other: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Size3D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Size3D<T, U>
fn mul_assign(&mut self, other: Scale<T, U, U>)
fn mul_assign(&mut self, other: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Vector2D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Vector2D<T, U>
fn mul_assign(&mut self, scale: Scale<T, U, U>)
fn mul_assign(&mut self, scale: Scale<T, U, U>)
*= operation. Read moreimpl<T, U> MulAssign<Scale<T, U, U>> for Vector3D<T, U>
impl<T, U> MulAssign<Scale<T, U, U>> for Vector3D<T, U>
fn mul_assign(&mut self, scale: Scale<T, U, U>)
fn mul_assign(&mut self, scale: Scale<T, U, U>)
*= operation. Read moreimpl<T, Src, Dst> Ord for Scale<T, Src, Dst>where
T: Ord,
impl<T, Src, Dst> Ord for Scale<T, Src, Dst>where
T: Ord,
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
impl<T, Src, Dst> PartialOrd for Scale<T, Src, Dst>where
T: PartialOrd,
impl<T, Src, Dst> PartialOrd for Scale<T, Src, Dst>where
T: PartialOrd,
impl<S> Transformation<S> for Scale<S, UnknownUnit, UnknownUnit>where
S: Scalar,
impl<S> Transformation<S> for Scale<S, UnknownUnit, UnknownUnit>where
S: Scalar,
fn transform_point(&self, p: Point2D<S, UnknownUnit>) -> Point2D<S, UnknownUnit>
fn transform_vector( &self, v: Vector2D<S, UnknownUnit>, ) -> Vector2D<S, UnknownUnit>
impl<T, Src, Dst> Copy for Scale<T, Src, Dst>where
T: Copy,
impl<T, Src, Dst> Eq for Scale<T, Src, Dst>where
T: Eq,